@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: doc2hlp.c,v 1.16 2005/06/03 05:11:55 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - doc2hlp.c */
 
 /*[
@@ -58,8 +54,8 @@ static char *RCSid() { return RCSid("$Id: doc2hlp.c,v 1.16 2005/06/03 05:11:55 s
 
 extern TBOOLEAN single_top_level;
 
-void convert __PROTO((FILE *, FILE *));
-void process_line __PROTO((char *, FILE *));
+void convert(FILE *, FILE *);
+void process_line(char *, FILE *);
 
 int
 main (int argc, char **argv)
@@ -123,6 +119,7 @@ process_line (char *line, FILE *b)
 	}
     case '=':			/* latex index entry */
     case 'F':			/* latex embedded figure */
+    case 'D':			/* html embedded figure */
     case '#':{			/* latex table entry */
 	    break;		/* ignore */
 	}
