@@ -1,7 +1,3 @@
-/*
- * $Id: datafile.h,v 1.49 2014/06/14 23:22:56 markisch Exp $
- */
-
 /* GNUPLOT - datafile.h */
 
 /*[
@@ -56,7 +52,8 @@ enum DF_STATUS {
     DF_FOUND_KEY_TITLE = -6,
     DF_KEY_TITLE_MISSING = -7,
     DF_STRINGDATA = -8,
-    DF_COLUMN_HEADERS = -9
+    DF_COLUMN_HEADERS = -9,
+    DF_COMPLEX_VALUE = -10
 };
 
 /* large file support (offsets potentially > 2GB) */
@@ -94,20 +91,29 @@ extern TBOOLEAN df_matrix;
 /* is this a binary file? */
 extern TBOOLEAN df_binary;
 
+/* will this need to be expanded into a fully populated grid? */
+extern TBOOLEAN df_sparse_matrix;
+
+/* was df_open called on something that turned out to be a voxel grid? */
+extern TBOOLEAN df_voxelgrid;
+
 extern char *df_filename;
 extern int df_line_number;
 extern AXIS_INDEX df_axis[];
 
-#ifdef BACKWARDS_COMPATIBLE
-extern struct udft_entry ydata_func; /* deprecated "thru" function */
-#endif
-
 /* Returned to caller by df_readline() */
 extern char *df_tokens[];
+extern struct value df_strings[];	/* used only by TABLESTYLE */
 
 /* number of columns in first row of data return to user in STATS_columns */
 extern int df_last_col;
 
+/* number of matrix elements entered as missing or NaN */
+extern int df_bad_matrix_values;
+
+/* First row of data is known to contain headers rather than data */
+extern TBOOLEAN df_columnheaders;
+
 /* string representing missing values, ascii datafiles */
 extern char *missing_val;
 
@@ -131,31 +137,40 @@ extern TBOOLEAN df_nofpe_trap;
 extern TBOOLEAN evaluate_inside_using;
 extern TBOOLEAN df_warn_on_missing_columnheader;
 
+/* Used by plot title columnhead, stats name columnhead */
+extern char *df_key_title;
+extern struct at_type *df_plot_title_at;
+
 /* Prototypes of functions exported by datafile.c */
 
-int df_open __PROTO((const char *, int, struct curve_points *));
-int df_readline __PROTO((double [], int));
-void df_close __PROTO((void));
-void df_showdata __PROTO((void));
-int df_2dbinary __PROTO((struct curve_points *));
-int df_3dmatrix __PROTO((struct surface_points *, int));
-void df_set_key_title __PROTO((struct curve_points *));
-void df_set_key_title_columnhead __PROTO((struct curve_points *));
-char * df_parse_string_field __PROTO((char *));
-int expect_string __PROTO((const char column ));
+int df_open(const char *, int, struct curve_points *);
+int df_readline(double [], int);
+void df_close(void);
+void df_init(void);
+char * df_fgets(FILE *);
+void df_showdata(void);
+int df_2dbinary(struct curve_points *);
+int df_3dmatrix(struct surface_points *, int);
+void df_set_key_title(struct curve_points *);
+void df_set_key_title_columnhead(struct curve_points *);
+char * df_parse_string_field(char *);
+int expect_string(const signed char column);
+void require_value(const char column);
 
-void df_reset_after_error __PROTO((void));
-void f_dollars __PROTO((union argument *x));
-void f_column  __PROTO((union argument *x));
-void f_columnhead  __PROTO((union argument *x));
-void f_valid   __PROTO((union argument *x));
-void f_timecolumn   __PROTO((union argument *x));
-void f_stringcolumn   __PROTO((union argument *x));
+char *df_retrieve_columnhead(int column);
+void df_reset_after_error(void);
+void f_dollars(union argument *x);
+void f_column (union argument *x);
+void f_columnhead (union argument *x);
+void f_valid  (union argument *x);
+void f_timecolumn  (union argument *x);
+void f_stringcolumn  (union argument *x);
 
 struct use_spec_s {
     int column;
     int expected_type;
     struct at_type *at;
+    int depends_on_column;
 };
 
 
@@ -261,31 +276,39 @@ typedef struct df_binary_file_record_struct {
     df_translation_type scan_trans;   /* translate via origin, center or default */
     double scan_cen_or_ori[3];        /* vector representing center or origin, x/y/z */
 
+    /* `matrix every ::lowx:lowy:` can select a submatrix.
+     * This is its size.
+     */
+    int submatrix_ncols;
+    int submatrix_nrows;
+
     /* *** Do not modify outside of datafile.c!!! *** */
-    char GPFAR *memory_data;
+    char *memory_data;
 } df_binary_file_record_struct;
 
 extern df_binary_file_record_struct *df_bin_record;
 extern int df_num_bin_records;
-extern struct coordinate blank_data_line;
+extern const struct coordinate blank_data_line;
 
 extern struct use_spec_s use_spec[];
 
 /* Prototypes of functions exported by datafile.c */
 
-void df_show_binary __PROTO((FILE *fp));
-void df_show_datasizes __PROTO((FILE *fp));
-void df_show_filetypes __PROTO((FILE *fp));
-void df_set_datafile_binary __PROTO((void)); 
-void df_unset_datafile_binary __PROTO((void));
-void df_add_binary_records __PROTO((int, df_records_type));
-void df_extend_binary_columns __PROTO((int));
-void df_set_skip_before __PROTO((int col, int bytes));                /* Number of bytes to skip before a binary column. */
+void df_show_binary(FILE *fp);
+void df_show_datasizes(FILE *fp);
+void df_show_filetypes(FILE *fp);
+void df_set_datafile_binary(void); 
+void df_unset_datafile_binary(void);
+void df_add_binary_records(int, df_records_type);
+void df_extend_binary_columns(int);
+void df_set_skip_before(int col, int bytes);                /* Number of bytes to skip before a binary column. */
 #define df_set_skip_after(col,bytes) df_set_skip_before(col+1,bytes)  /* Number of bytes to skip after a binary column. */
-void df_set_read_type __PROTO((int col, df_data_type type));          /* Type of data in the binary column. */
-df_data_type df_get_read_type __PROTO((int col));                     /* Type of data in the binary column. */
-int df_get_read_size __PROTO((int col));                              /* Size of data in the binary column. */
-int df_get_num_matrix_cols __PROTO((void));
-void df_set_plot_mode __PROTO((int));
+void df_set_read_type(int col, df_data_type type);          /* Type of data in the binary column. */
+df_data_type df_get_read_type(int col);                     /* Type of data in the binary column. */
+int df_get_read_size(int col);                              /* Size of data in the binary column. */
+int df_get_num_matrix_cols(void);
+void df_set_plot_mode(int);
+
+void populate_sparse_matrix(struct coordinate **points, int *p_count);
 
 #endif /* GNUPLOT_DATAFILE_H */
