@@ -47,13 +47,14 @@
 enum operators {
     /* keep this in line with table in eval.c */
     PUSH, PUSHC, PUSHD1, PUSHD2, PUSHD, POP,
-    CALL, CALLN, SUM, LNOT, BNOT, UMINUS,
+    CALL, CALLN, SUM, LNOT, BNOT, UMINUS, NOP,
     LOR, LAND, BOR, XOR, BAND, EQ, NE, GT, LT, GE, LE, 
     LEFTSHIFT, RIGHTSHIFT, PLUS, MINUS,
     MULT, DIV, MOD, POWER, FACTORIAL, BOOLE,
     DOLLARS,
     CONCATENATE, EQS, NES, RANGE, INDEX, CARDINALITY,
     ASSIGN,
+    EVAL,
     /* only jump operators go between jump and sf_start, for is_jump() */
     JUMP, JUMPZ, JUMPNZ, JTERN, SF_START,
 
@@ -63,7 +64,8 @@ enum operators {
 #endif
 
     /* functions specific to using spec */
-    COLUMN, STRINGCOLUMN, STRCOL, COLUMNHEAD
+    COLUMN, STRINGCOLUMN, STRCOL, COLUMNHEAD, COLUMNHEADER,
+    VALID, TIMECOLUMN
 };
 #define is_jump(operator) \
     ((operator) >=(int)JUMP && (operator) <(int)SF_START)
@@ -132,6 +134,7 @@ extern const struct ft_entry ft[]; /* The table of builtin functions */
 extern struct udft_entry *first_udf; /* user-def'd functions */
 extern struct udvt_entry *first_udv; /* user-def'd variables */
 extern struct udvt_entry udv_pi; /* 'pi' variable */
+extern struct udvt_entry *udv_I; /* 'I' (sqrt(-1)) */
 extern struct udvt_entry *udv_NaN; /* 'NaN' variable */
 extern struct udvt_entry **udv_user_head; /* first udv that can be deleted */
 extern TBOOLEAN undefined;
@@ -154,6 +157,10 @@ struct value * pop_or_convert_from_string(struct value *);
 void free_value(struct value *a);
 void gpfree_string(struct value *a);
 void gpfree_array(struct value *a);
+void make_array_permanent(struct value *a);
+struct value * array_slice(struct value *full, int beg, int end);
+
+void init_array( struct udvt_entry *array, int size );
 
 void reset_stack(void);
 void check_stack(void);
@@ -171,8 +178,10 @@ void f_jtern(union argument *x);
 void execute_at(struct at_type *at_ptr);
 void evaluate_at(struct at_type *at_ptr, struct value *val_ptr);
 void real_free_at(struct at_type *at_ptr);
+void free_action_entry(struct at_entry *a);
 struct udvt_entry * add_udv_by_name(char *key);
 struct udvt_entry * get_udv_by_name(char *key);
+struct udft_entry * get_udf_by_token(int token);
 void del_udv_by_name( char *key, TBOOLEAN isWildcard );
 void clear_udf_list(void);
 
