@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: gadgets.c,v 1.115.2.5 2016/02/05 05:09:10 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - gadgets.c */
 
 /*[
@@ -45,33 +41,37 @@ static char *RCSid() { return RCSid("$Id: gadgets.c,v 1.115.2.5 2016/02/05 05:09
 #include "pm3d.h"
 
 /* This file contains mainly a collection of global variables that
- * used to be in 'set.c', where they didn't really belong. They
  * describe the status of several parts of the gnuplot plotting engine
  * that are used by both 2D and 3D plots, and thus belong neither to
  * graphics.c nor graph3d.c, alone. This is not a very clean solution,
  * but better than mixing internal status and the user interface as we
  * used to have it, in set.c and setshow.h */
 
-legend_key keyT = DEFAULT_KEY_PROPS;
+legend_key keyT;
 
 /* Description of the color box associated with CB_AXIS */
 color_box_struct color_box; /* initialized in init_color() */
 color_box_struct default_color_box = {SMCOLOR_BOX_DEFAULT, 'v', 1, LT_BLACK, LAYER_FRONT, 0,
 					{screen, screen, screen, 0.90, 0.2, 0.0},
-					{screen, screen, screen, 0.05, 0.6, 0.0},
+					{screen, screen, screen, 0.05, 0.6, 0.0}, FALSE,
 					{0,0,0,0} };
 
-/* The graph box, in terminal coordinates, as calculated by boundary()
- * or boundary3d(): */
+/* The graph box (terminal coordinates) calculated by boundary() or boundary3d() */
 BoundingBox plot_bounds;
 
+/* The bounding box for 3D plots prior to applying view transformations */
+BoundingBox page_bounds;
+
+/* The region within a multiplot belonging to the current plot */
+BoundingBox active_bounds;
+
 /* The bounding box for the entire drawable area  of current terminal */
 BoundingBox canvas;
 
 /* The bounding box against which clipping is to be done */
 BoundingBox *clip_area = &plot_bounds;
 
-/* 'set size', 'set origin' setttings */
+/* 'set size', 'set origin' settings */
 float xsize = 1.0;		/* scale factor for size */
 float ysize = 1.0;		/* scale factor for size */
 float zsize = 1.0;		/* scale factor for size */
@@ -86,9 +86,9 @@ int aspect_ratio_3D = 0;	/* 2 will put x and y on same scale, 3 for z also */
 t_position lmargin = DEFAULT_MARGIN_POSITION;
 /* space between bottom and plot_bounds.ybot in chars (-1: computed) */
 t_position bmargin = DEFAULT_MARGIN_POSITION;
-/* space between right egde and plot_bounds.xright in chars (-1: computed) */
+/* space between right edge and plot_bounds.xright in chars (-1: computed) */
 t_position rmargin = DEFAULT_MARGIN_POSITION;
-/* space between top egde and plot_bounds.ytop in chars (-1: computed) */
+/* space between top edge and plot_bounds.ytop in chars (-1: computed) */
 t_position tmargin = DEFAULT_MARGIN_POSITION;
 
 /* Pointer to first 'set dashtype' definition in linked list */
@@ -105,27 +105,36 @@ struct linestyle_def *first_mono_linestyle = NULL;
 /* Pointer to first 'set style arrow' definition in linked list */
 struct arrowstyle_def *first_arrowstyle = NULL;
 
+/* Listhead for pixmaps */
+t_pixmap *pixmap_listhead = NULL;
+
 /* Holds the properties from 'set style parallelaxis' */
 struct pa_style parallel_axis_style = DEFAULT_PARALLEL_AXIS_STYLE;
 
+/* Holds properties for 'set style spiderplot' */
+struct spider_web spiderplot_style = DEFAULT_SPIDERPLOT_STYLE;
+
 /* set arrow */
 struct arrow_def *first_arrow = NULL;
 
-#ifdef EAM_OBJECTS
 /* Pointer to first object instance in linked list */
 struct object *first_object = NULL;
-#endif
+/* Pointer to array of grid walls */
+struct object grid_wall[5] = {WALL_Y0, WALL_X0, WALL_Y1, WALL_X1, WALL_Z0};
 
 /* 'set title' status */
 text_label title = EMPTY_LABELSTRUCT;
 
 /* 'set timelabel' status */
 text_label timelabel = EMPTY_LABELSTRUCT;
-int timelabel_rotate = FALSE;
 int timelabel_bottom = TRUE;
 
 /* flag for polar mode */
 TBOOLEAN polar = FALSE;
+TBOOLEAN inverted_raxis = FALSE;
+
+/* toggle spiderplot mode on/off */
+TBOOLEAN spiderplot = FALSE;
 
 /* zero threshold, may _not_ be 0! */
 double zero = ZERO;
@@ -134,21 +143,31 @@ double zero = ZERO;
 double pointsize = 1.0;
 double pointintervalbox = 1.0;
 
+/* used for filled points */
+t_colorspec background_fill = BACKGROUND_COLORSPEC;
+
+/* box width (automatic) for plot style "with boxes" */
+double   boxwidth              = -1.0;
+/* whether box width is absolute (default) or relative */
+TBOOLEAN boxwidth_is_absolute  = TRUE;
+
 /* set border */
 int draw_border = 31;	/* The current settings */
 int user_border = 31;	/* What the user last set explicitly */
 int border_layer = LAYER_FRONT;
-# define DEFAULT_BORDER_LP { 0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 1.0, 1.0, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN }
+# define DEFAULT_BORDER_LP { 0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 0, 1.0, 1.0, DEFAULT_P_CHAR, BLACK_COLORSPEC, DEFAULT_DASHPATTERN }
 struct lp_style_type border_lp = DEFAULT_BORDER_LP;
 const struct lp_style_type default_border_lp = DEFAULT_BORDER_LP;
-const struct lp_style_type background_lp = {0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN};
+const struct lp_style_type background_lp = {0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN};
+TBOOLEAN cornerpoles = TRUE;
 
 /* set clip */
 TBOOLEAN clip_lines1 = TRUE;
 TBOOLEAN clip_lines2 = FALSE;
 TBOOLEAN clip_points = FALSE;
+TBOOLEAN clip_radial = FALSE;
 
-static int clip_line __PROTO((int *, int *, int *, int *));
+static int clip_line(int *, int *, int *, int *);
 
 /* set samples */
 int samples_1 = SAMPLES;
@@ -161,6 +180,7 @@ enum PLOT_STYLE data_style = POINTSTYLE;
 enum PLOT_STYLE func_style = LINES;
 
 TBOOLEAN parametric = FALSE;
+TBOOLEAN in_parametric = FALSE;
 
 /* If last plot was a 3d one. */
 TBOOLEAN is_3d_plot = FALSE;
@@ -174,19 +194,16 @@ TBOOLEAN volatile_data = FALSE;
 
 fill_style_type default_fillstyle = { FS_EMPTY, 100, 0, DEFAULT_COLORSPEC } ;
 
-#ifdef EAM_OBJECTS
 /* Default rectangle style - background fill, black border */
 struct object default_rectangle = DEFAULT_RECTANGLE_STYLE;
 struct object default_circle = DEFAULT_CIRCLE_STYLE;
 struct object default_ellipse = DEFAULT_ELLIPSE_STYLE;
-#endif
 
 /* filledcurves style options */
 filledcurves_opts filledcurves_opts_data = EMPTY_FILLEDCURVES_OPTS;
 filledcurves_opts filledcurves_opts_func = EMPTY_FILLEDCURVES_OPTS;
 
-#if TRUE || defined(BACKWARDS_COMPATIBLE)
-/* Prefer line styles over plain line types */
+#ifdef BACKWARD_COMPATIBILITY
 TBOOLEAN prefer_line_styles = FALSE;
 #endif
 
@@ -200,9 +217,15 @@ boxplot_style boxplot_opts = DEFAULT_BOXPLOT_STYLE;
 /* WINDOWID to be filled by terminals running on X11 (x11, wxt, qt, ...) */
 int current_x11_windowid = 0;
 
-#ifdef EAM_BOXED_TEXT
-textbox_style textbox_opts = DEFAULT_TEXTBOX_STYLE;
-#endif
+textbox_style textbox_opts[NUM_TEXTBOX_STYLES];
+
+/*
+ * Image data or pm3d quadrangles can be masked by first loading a set
+ * of masking polygons via dummy plotting style "with mask".
+ */
+struct iso_curve *mask_2Dpolygon_set = NULL;
+struct iso_curve *mask_3Dpolygon_set = NULL;
+static TBOOLEAN interior(double x, double y, struct coordinate *mask, int nv);
 
 /*****************************************************************/
 /* Routines that deal with global objects defined in this module */
@@ -219,7 +242,7 @@ textbox_style textbox_opts = DEFAULT_TEXTBOX_STYLE;
  * 0 is returned if inside.
  */
 int
-clip_point(unsigned int x, unsigned int y)
+clip_point(int x, int y)
 {
     int ret_val = 0;
 
@@ -241,18 +264,19 @@ clip_point(unsigned int x, unsigned int y)
  *   This routine uses the cohen & sutherland bit mapping for fast clipping -
  * see "Principles of Interactive Computer Graphics" Newman & Sproull page 65.
  */
-void
+int
 draw_clip_line(int x1, int y1, int x2, int y2)
 {
     struct termentry *t = term;
+    int state;
 
-    if (!clip_line(&x1, &y1, &x2, &y2))
-	/* clip_line() returns zero --> segment completely outside
-	 * bounding box */
-	return;
+    state = clip_line(&x1, &y1, &x2, &y2);
+    if (state != 0) {
+	(*t->move) (x1, y1);
+	(*t->vector) (x2, y2);
+    }
 
-    (*t->move) (x1, y1);
-    (*t->vector) (x2, y2);
+    return state;
 }
 
 /* Draw a contiguous line path which may be clipped. Compared to
@@ -266,16 +290,22 @@ draw_clip_polygon(int points, gpiPoint *p)
     int x1, y1, x2, y2;
     int pos1, pos2, clip_ret;
     struct termentry *t = term;
+    TBOOLEAN continuous = TRUE;
 
     if (points <= 1) 
 	return;
 
+    if (p[0].x != p[points-1].x || p[0].y != p[points-1].y)
+	continuous = FALSE;
+
     x1 = p[0].x;
     y1 = p[0].y;
     pos1 = clip_point(x1, y1);
     if (!pos1) /* move to first point if it is inside */
 	(*t->move)(x1, y1);
 
+    newpath();
+
     for (i = 1; i < points; i++) {
 	x2 = p[i].x;
 	y2 = p[i].y;
@@ -287,6 +317,9 @@ draw_clip_polygon(int points, gpiPoint *p)
 	    if (pos1) /* first vertex was recalculated, move to new start point */
 		(*t->move)(x1, y1);
 	    (*t->vector)(x2, y2);
+	} else {
+	    /* Path is not continuous; make sure closepath is not called */
+	    continuous = FALSE;
 	}
 
 	x1 = p[i].x;
@@ -294,29 +327,88 @@ draw_clip_polygon(int points, gpiPoint *p)
 	/* The end point and the line do not necessarily have the same
 	 * status. The end point can be 'inside', but the whole line is
 	 * 'outside'. Do not update pos1 in this case.  Bug #1268.
-	 * FIXME: This is papering over an inconsistency in coordinate
-	 * calculation somewhere else!
 	 */
 	if (!(clip_ret == 0 && pos2 == 0))
 	    pos1 = pos2;
     }
+
+    /* Only call closepath if the polygon is truly closed; otherwise */
+    /* a spurious line connecting the start and end is generated.    */
+    if (continuous)
+	closepath();
 }
 
+/*
+ * arrow is specified in terminal coordinates
+ * but we use double rather than int so that the precision is sufficient
+ * to orient and draw the arrow head correctly even for very short vectors.
+ */
 void
-draw_clip_arrow( int sx, int sy, int ex, int ey, int head)
+draw_clip_arrow( double dsx, double dsy, double dex, double dey, t_arrow_head head)
 {
     struct termentry *t = term;
 
+    int sx = axis_map_toint(dsx);
+    int sy = axis_map_toint(dsy);
+    int ex = axis_map_toint(dex);
+    int ey = axis_map_toint(dey);
+    int dx, dy;
+
     /* Don't draw head if the arrow itself is clipped */
     if (clip_point(sx,sy))
 	head &= ~BACKHEAD;
     if (clip_point(ex,ey))
 	head &= ~END_HEAD;
-    clip_line(&sx, &sy, &ex, &ey);
 
-    /* Call terminal routine to draw the clipped arrow */
-    (*t->arrow)((unsigned int)sx, (unsigned int)sy,
-		(unsigned int)ex, (unsigned int)ey, head);
+    /* clip_line returns 0 if the whole thing is out of range */
+    if (!clip_line(&sx, &sy, &ex, &ey))
+	return;
+
+    /* Special case code for short vectors */
+    /* Most terminals are OK with using this code for long vectors also.	*/
+    /* However some terminals (e.g. tikz) look terrible because the shaft of a	*/
+    /* long vector overruns the head when the head is drawn with HEADS_ONLY.	*/
+    /* FIXME:  this is a very ad hoc definition of "short".			*/
+    dx = abs(ex-sx);
+    dy = abs(ey-sy);
+    if (dx < 25 && dy < 25) {
+
+	/* draw the body of the vector (rounding errors are a problem) */
+	if (dx > 1 || dy > 1)
+	    if (!((t->flags & TERM_IS_LATEX)))
+		(*t->arrow)(sx, sy, ex, ey, SHAFT_ONLY | head);
+
+	/* if we're not supposed to be drawing any heads, we're done */
+	if ((head & BOTH_HEADS) == NOHEAD)
+	    return;
+
+	/* If this is truly a 0-vector, then we CAN'T draw the head */
+	/* because the orientation of the head is indeterminate     */
+	if (dsx == dex && dsy == dey)
+	    return;
+
+	/* If the head size is fixed we are free to change the length of */
+	/* very short vectors so that the orientation is accurate.	 */
+	if (curr_arrow_headfixedsize) {
+	    /* Direction vector in (dex,dey). I need to convert this to integers
+	     * with a scale that's large-enough to give me good angular resolution,
+	     * but small-enough to not overflow the data type.
+	     */
+	    double rescale = 1000. / GPMAX( fabs(dex-dsx), fabs(dey-dsy) );
+	    int newlenx = (dex - dsx) * rescale;
+	    int newleny = (dey - dsy) * rescale;
+
+	    if (head & END_HEAD)
+		(*t->arrow)(ex - newlenx, ey - newleny, ex, ey, END_HEAD|HEADS_ONLY);
+	    if (head & BACKHEAD)
+		(*t->arrow)(sx, sy, sx + newlenx, sy + newleny, BACKHEAD|HEADS_ONLY);
+	} else {
+	    (*t->arrow)(sx, sy, ex, ey, head|HEADS_ONLY);
+	}
+    } else {
+	/* The normal case, draw the whole thing at once */
+	(*t->arrow)(sx, sy, ex, ey, head);
+    }
 }
 
 /* Clip the given line to drawing coords defined by BoundingBox.
@@ -338,8 +430,7 @@ clip_line(int *x1, int *y1, int *x2, int *y2)
      * This is now addressed by making dx and dy (double) rather than (int)
      * but it might be better to hard-code the sign tests.
      */
-    double dx, dy;
-    double x, y;
+    double dx, dy, x, y;
 
     int x_intr[4], y_intr[4], count, pos1, pos2;
     int x_max, x_min, y_max, y_min;
@@ -356,8 +447,8 @@ clip_line(int *x1, int *y1, int *x2, int *y2)
      * when the line passes directly through at least one corner).
      */
     count = 0;
-    dx = *x2 - *x1;
-    dy = *y2 - *y1;
+    dx = (double)(*x2) - (double)(*x1);
+    dy = (double)(*y2) - (double)(*y1);
     /* Find intersections with the x parallel bbox lines: */
     if (dy != 0) {
 	x = (clip_area->ybot - *y2) * dx / dy + *x2;	/* Test for clip_area->ybot boundary. */
@@ -423,11 +514,6 @@ clip_line(int *x1, int *y1, int *x2, int *y2)
 	    *y2 = y_intr[1];
 	}
     } else if (pos1) {		/* Only x1/y1 was out - update only it */
-	/* Nov 2010: When clip_line() and draw_clip_line() were consolidated in */
-	/* 2000, the test below was the only point of difference between them.  */
-	/* Unfortunately, the wrong version was kept. Now I change it back.     */
-	/* The effect of the wrong version (>= rather than >) was that a line   */
-	/* from ymin to ymax+eps was clipped to ymin,ymin rather than ymin,ymax */
 	if (dx * (*x2 - x_intr[0]) + dy * (*y2 - y_intr[0]) > 0) {
 	    *x1 = x_intr[0];
 	    *y1 = y_intr[0];
@@ -472,16 +558,22 @@ vertex_is_inside(gpiPoint test_vertex, gpiPoint *clip_boundary)
 void
 intersect_polyedge_with_boundary(gpiPoint first, gpiPoint second, gpiPoint *intersect, gpiPoint *clip_boundary)
 {
-    /* this routine is called only if one point is outside and the other
-       is inside, which implies that clipping is needed at a horizontal
-       boundary, that second.y is different from first.y and no division
-       by zero occurs. Same for vertical boundary and x coordinates. */
+    /* This routine is called only if one point is outside and the other
+     * is inside, which implies that clipping is needed at a horizontal
+     * boundary, that second.y is different from first.y and no division
+     * by zero occurs. Same for vertical boundary and x coordinates.
+     * Conversion to double is needed to prevent integer overflow.
+     */
+    double dx = (second.x - first.x);
+    double dy = (second.y - first.y);
     if (clip_boundary[0].y == clip_boundary[1].y) { /* horizontal */
 	(*intersect).y = clip_boundary[0].y;
-	(*intersect).x = first.x + (clip_boundary[0].y - first.y) * (second.x - first.x)/(second.y - first.y);
+	(*intersect).x = first.x
+			+ (double)(clip_boundary[0].y - first.y) * dx/dy;
     } else { /* vertical */
 	(*intersect).x = clip_boundary[0].x;
-	(*intersect).y = first.y + (clip_boundary[0].x - first.x) * (second.y - first.y)/(second.x - first.x);
+	(*intersect).y = first.y
+			+ (double)(clip_boundary[0].x - first.x) * dy/dx;
     }
 }
 
@@ -535,12 +627,12 @@ clip_polygon(gpiPoint *in, gpiPoint *out, int in_length, int *out_length)
     gpiPoint clip_boundary[5];
     static gpiPoint *tmp_corners = NULL;
 
-    if (!clip_area) {
+    if (!clip_area || in_length < 3) {
 	memcpy(out, in, in_length * sizeof(gpiPoint));
 	*out_length = in_length;
 	return;
     }
-    tmp_corners = gp_realloc(tmp_corners, 2 * in_length * sizeof(gpiPoint), "clip_polygon");
+    tmp_corners = gp_realloc(tmp_corners, 4 * in_length * sizeof(gpiPoint), "clip_polygon");
 
     /* vertices of the rectangular clipping window starting from
        top-left in counterclockwise direction */
@@ -555,7 +647,7 @@ clip_polygon(gpiPoint *in, gpiPoint *out, int in_length, int *out_length)
     clip_boundary[4] = clip_boundary[0];
 
     memcpy(tmp_corners, in, in_length * sizeof(gpiPoint));
-    for(i = 0; i < 4; i++) {
+    for (i = 0; i < 4; i++) {
 	clip_polygon_to_boundary(tmp_corners, out, in_length, out_length, clip_boundary+i);
 	memcpy(tmp_corners, out, *out_length * sizeof(gpiPoint));
 	in_length = *out_length;
@@ -563,32 +655,125 @@ clip_polygon(gpiPoint *in, gpiPoint *out, int in_length, int *out_length)
 }
 
 /* Two routines to emulate move/vector sequence using line drawing routine. */
-static unsigned int move_pos_x, move_pos_y;
+static int move_pos_x, move_pos_y;
 
 void
-clip_move(unsigned int x, unsigned int y)
+clip_move(int x, int y)
 {
     move_pos_x = x;
     move_pos_y = y;
 }
 
 void
-clip_vector(unsigned int x, unsigned int y)
+clip_vector(int x, int y)
 {
     draw_clip_line(move_pos_x, move_pos_y, x, y);
     move_pos_x = x;
     move_pos_y = y;
 }
 
+/*
+ * draw_polar_clip_line() assumes that the endpoints have already
+ * been categorized as INRANGE/OUTRANGE, and that "set clip radial"
+ * is in effect.
+ */
+void
+draw_polar_clip_line( double xbeg, double ybeg, double xend, double yend)
+{
+    double R;			/* radius of limiting circle */
+    double a, b;		/* line expressed as y = a*x + b */
+    double x1, y1, x2, y2;	/* Intersections of line and circle */
+    double Q, Q2;		/* sqrt term of quadratic equation */
+    TBOOLEAN vertical = FALSE;	/* flag for degenerate case */
+    TBOOLEAN beg_inrange, end_inrange;
+
+    if (R_AXIS.set_max == -VERYLARGE)
+	goto outside;
+    R = R_AXIS.set_max - R_AXIS.set_min;
+
+    /* If both endpoints are inside the limiting circle, draw_clip_line suffices */
+    beg_inrange = (xbeg*xbeg + ybeg*ybeg) <= R*R;
+    end_inrange = (xend*xend + yend*yend) <= R*R;
+    if (beg_inrange && end_inrange) {
+	draw_clip_line(map_x(xbeg), map_y(ybeg), map_x(xend), map_y(yend));
+	return;
+    }
+
+    /* FIXME:  logscale and other odd cases are not covered by this equation */
+    if (fabs(xbeg - xend) > ZERO) {
+	/* Recast line in the form y = a*x + b */
+	a = (yend - ybeg) / (xend - xbeg);
+	b = ybeg - xbeg * a;
+	/* the line may intersect a circle of radius R in two places */
+	Q2 = 4*a*a*b*b - 4 * (1 + a*a) * (b*b - R*R);
+	if (Q2 < 0)
+	    goto outside;
+	Q = sqrt(Q2);
+	x1 = (-2*a*b + Q) / ( 2*(1+a*a));
+	x2 = (-2*a*b - Q) / ( 2*(1+a*a));
+	y1 = a * x1 + b;
+	y2 = a * x2 + b;
+    } else {
+	/* degenerate case (vertical line) */
+	x1 = x2 = xbeg;
+	if (fabs(x1) > R)
+	    goto outside;
+	vertical = TRUE;
+	y1 = sqrt(R*R - x1*x1);
+	y2 = -y1;
+	if (!inrange(y1,ybeg,yend) && !inrange(y2,ybeg,yend))
+	    goto outside;
+    }
+
+    /* If one of the original endpoints was INRANGE then use it */
+    /* rather than the second intersection point.               */
+    if (vertical) {
+	y1 = GPMIN(y1, GPMAX(ybeg, yend));
+	y2 = GPMAX(y2, GPMIN(ybeg, yend));
+    } else if (beg_inrange) {
+	if (!inrange(x1, xbeg, xend)) {
+	    x1 = xbeg;
+	    y1 = ybeg;
+	} else {
+	    x2 = xbeg;
+	    y2 = ybeg;
+	}
+    } else if (end_inrange) {
+	if (!inrange(x1, xbeg, xend)) {
+	    x1 = xend;
+	    y1 = yend;
+	} else {
+	    x2 = xend;
+	    y2 = yend;
+	}
+    } else {
+	/* Both OUTRANGE. Are they on the same side of the circle? */
+	if (!inrange(x1, xbeg, xend))
+	    goto outside;
+    }
+
+    /* Draw the part of the line inside the bounding circle */
+    (term->move)(map_x(x1), map_y(y1));
+    (term->vector)(map_x(x2), map_y(y2));
+    /* fall through */
+
+outside:
+    /* Leave current position at unclipped endpoint */
+    (term->move)(map_x(xend), map_y(yend));
+    return;
+}
+
 /* Common routines for setting text or line color from t_colorspec */
 
 void
-apply_pm3dcolor(struct t_colorspec *tc, const struct termentry *t)
+apply_pm3dcolor(struct t_colorspec *tc)
 {
+    struct termentry *t = term;
+    double cbval;
+
     /* V5 - term->linetype(LT_BLACK) would clobber the current	*/
     /* dashtype so instead we use term->set_color(black).	*/
     static t_colorspec black = BLACK_COLORSPEC; 
-    double cbval;
 
     /* Replace colorspec with that of the requested line style */
     struct lp_style_type style;
@@ -602,36 +787,35 @@ apply_pm3dcolor(struct t_colorspec *tc, const struct termentry *t)
 	return;
     }
     if (tc->type == TC_LT) {
-	/* Removed Jan 2015 
-	if (!monochrome_terminal)
-	 */
-	    t->set_color(tc);
+	t->set_color(tc);
 	return;
     }
     if (tc->type == TC_RGB) {
-	/* FIXME: several plausible ways for monochrome terminals to handle color request
+	/* There are several plausible ways monochrome terminals might handle color request
 	 * (1) Allow all color requests despite the label "monochrome"
 	 * (2) Choose any color you want so long as it is black
 	 * (3) Convert colors to gray scale (NTSC?)
+	 *
+	 * We go for a modified (1):
+	 * Monochrome terminals are still allowed to display rgb variable colors.
 	 */
-	/* Monochrome terminals are still allowed to display rgb variable colors */
 	if (monochrome_terminal && tc->value >= 0)
 	    t->set_color(&black);
 	else
 	    t->set_color(tc);
 	return;
     }
-    if (!is_plot_with_palette()) {
-	t->set_color(&black);
+    /* Leave unchanged. (used only by "set errorbars"??) */
+    if (tc->type == TC_VARIABLE)
 	return;
-    }
+
     switch (tc->type) {
 	case TC_Z:
-		set_color(cb2gray(z2cb(tc->value)));
+		set_color(cb2gray(tc->value));
 		break;
 	case TC_CB:
 		if (CB_AXIS.log)
-		    cbval = (tc->value <= 0) ? CB_AXIS.min : (log(tc->value) / CB_AXIS.log_base);
+		    cbval = (tc->value <= 0) ? CB_AXIS.min : tc->value;
 		else
 		    cbval = tc->value;
 		set_color(cb2gray(cbval));
@@ -639,21 +823,25 @@ apply_pm3dcolor(struct t_colorspec *tc, const struct termentry *t)
 	case TC_FRAC:
 		set_color(sm_palette.positive == SMPAL_POSITIVE ?  tc->value : 1-tc->value);
 		break;
+	default:
+		break; /* cannot happen */
     }
 }
 
 void
-reset_textcolor(const struct t_colorspec *tc, const struct termentry *t)
+reset_textcolor(const struct t_colorspec *tc)
 {
     if (tc->type != TC_DEFAULT)
-	(*t->linetype)(LT_BLACK);
+	term->linetype(LT_BLACK);
 }
 
 
 void
 default_arrow_style(struct arrow_style_type *arrow)
 {
-    static const struct lp_style_type tmp_lp_style = DEFAULT_LP_STYLE_TYPE;
+    static const struct lp_style_type tmp_lp_style = 
+	{0, LT_SOLID, 0, DASHTYPE_SOLID, 0, 0, 1.0, 0.0, DEFAULT_P_CHAR,
+	DEFAULT_COLORSPEC, DEFAULT_DASHPATTERN};
 
     arrow->tag = -1;
     arrow->layer = LAYER_BACK;
@@ -689,20 +877,41 @@ apply_head_properties(struct arrow_style_type *arrow_properties)
     }
 }
 
+/* Used by arrow-drawing code in graphics.c graph3d.c */
+double effective_aspect_ratio(void)
+{
+#ifdef _WIN32
+    if (strcmp(term->name, "windows") == 0) {
+	double xscale = (axis_array[FIRST_X_AXIS].term_upper - axis_array[FIRST_X_AXIS].term_lower)
+			/ (axis_array[FIRST_X_AXIS].max - axis_array[FIRST_X_AXIS].min);
+	double yscale = (axis_array[FIRST_Y_AXIS].term_upper - axis_array[FIRST_Y_AXIS].term_lower)
+			/ (axis_array[FIRST_Y_AXIS].max - axis_array[FIRST_Y_AXIS].min);
+	double aspect_ratio = fabs(yscale/xscale);
+	return aspect_ratio;
+    }
+#endif
+
+    return (double)term->v_tic / (double)term->h_tic;
+}
+
 void
 free_labels(struct text_label *label)
 {
 struct text_label *temp;
 char *master_font = label->font;
 
-    /* Labels generated by 'plot with labels' all use the same font */
-    if (master_font)
-    	free(master_font);
+    /* Labels generated by 'plot with labels' all use the same font.
+     * Free it once and then skip the duplicate appearances below.
+     * Watchpoint labels also all use the same font, taken from the style,
+     * but that controlling copy should not be freed here.
+     */
+    if (label->tag == LABEL_TAG_PLOTLABELS || label->tag == LABEL_TAG_VARIABLE_ROTATE)
+	free(master_font);
 
     while (label) {
 	if (label->text)
 	    free(label->text);
-	if (label->font && label->font != master_font)
+	if (label->font != master_font)
 	    free(label->font);
 	temp=label->next;
 	free(label);
@@ -714,12 +923,11 @@ char *master_font = label->font;
 void
 get_offsets(
     struct text_label *this_label,
-    struct termentry *t,
     int *htic, int *vtic)
 {
     if ((this_label->lp_properties.flags & LP_SHOW_POINTS)) {
-	*htic = (pointsize * t->h_tic * 0.5);
-	*vtic = (pointsize * t->v_tic * 0.5);
+	*htic = (pointsize * term->h_tic * 0.5);
+	*vtic = (pointsize * term->v_tic * 0.5);
     } else {
 	*htic = 0;
 	*vtic = 0;
@@ -743,32 +951,37 @@ get_offsets(
  * This routine is used for both 2D and 3D plots.
  */
 void
-write_label(unsigned int x, unsigned int y, struct text_label *this_label)
+write_label(int x, int y, struct text_label *this_label)
 {
 	int htic, vtic;
 	int justify = JUST_TOP;	/* This was the 2D default; 3D had CENTRE */
+	textbox_style *textbox = NULL;
 
-	apply_pm3dcolor(&(this_label->textcolor),term);
+	apply_pm3dcolor(&(this_label->textcolor));
 	ignore_enhanced(this_label->noenhanced);
 
 	/* The text itself */
 	if (this_label->hypertext) {
-	    /* Treat text as hypertext */
-	    char *font = this_label->font;
-	    if (font)
-		term->set_font(font);
-	    if (term->hypertext)
-	        term->hypertext(TERM_HYPERTEXT_TOOLTIP, this_label->text);
-	    if (font)
-		term->set_font("");
+	    if (this_label->text && *(this_label->text)) {
+		/* Treat text as hypertext */
+		char *font = this_label->font;
+		if (font)
+		    term->set_font(font);
+		if (term->hypertext)
+		    term->hypertext(TERM_HYPERTEXT_TOOLTIP, this_label->text);
+		if (font)
+		    term->set_font("");
+	    }
 	} else {
 	    /* A normal label (always print text) */
-	    get_offsets(this_label, term, &htic, &vtic);
-#ifdef EAM_BOXED_TEXT
+	    get_offsets(this_label, &htic, &vtic);
+	    if (this_label->boxed < 0)
+		textbox = &textbox_opts[0];
+	    else if (this_label->boxed > 0)
+		textbox = &textbox_opts[this_label->boxed];
 	    /* Initialize the bounding box accounting */
-	    if (this_label->boxed && term->boxed_text)
+	    if (textbox && term->boxed_text && (textbox->opaque || !textbox->noborder))
 		(*term->boxed_text)(x + htic, y + vtic, TEXTBOX_INIT);
-#endif
 	    if (this_label->rotate && (*term->text_angle) (this_label->rotate)) {
 		write_multiline(x + htic, y + vtic, this_label->text,
 				this_label->pos, justify, this_label->rotate,
@@ -779,40 +992,57 @@ write_label(unsigned int x, unsigned int y, struct text_label *this_label)
 				this_label->pos, justify, 0, this_label->font);
 	    }
 	}
-#ifdef EAM_BOXED_TEXT
-	/* Adjust the bounding box margins */
-	if (this_label->boxed && term->boxed_text)
-	    (*term->boxed_text)((int)(textbox_opts.xmargin * 100.),
-		(int)(textbox_opts.ymargin * 100.), TEXTBOX_MARGINS);
+	if (textbox && term->boxed_text && (textbox->opaque || !textbox->noborder))
+	{
+
+	    /* Adjust the bounding box margins */
+	    (*term->boxed_text)((int)(textbox->xmargin * 100.),
+		(int)(textbox->ymargin * 100.), TEXTBOX_MARGINS);
 
-	if (this_label->boxed && term->boxed_text && textbox_opts.opaque) {
 	    /* Blank out the box and reprint the label */
-	    (*term->boxed_text)(0,0, TEXTBOX_BACKGROUNDFILL);
-	    if (this_label->rotate && (*term->text_angle) (this_label->rotate)) {
-		write_multiline(x + htic, y + vtic, this_label->text,
-			    this_label->pos, justify, this_label->rotate,
-			    this_label->font);
-		(*term->text_angle) (0);
-	    } else {
-		write_multiline(x + htic, y + vtic, this_label->text,
-			    this_label->pos, justify, 0, this_label->font);
+	    if (textbox->opaque) {
+		apply_pm3dcolor(&textbox->fillcolor);
+		(*term->boxed_text)(0,0, TEXTBOX_BACKGROUNDFILL);
+		/* The epslatex/cairolatex terminals reuse a previously saved textbox
+		 * so explicitly re-writing it here is at best redundant.
+		 */
+		if ((term->flags & TERM_REUSES_BOXTEXT) == 0) {
+		    apply_pm3dcolor(&(this_label->textcolor));
+		    /* Init for each of fill and border */
+		    if (!textbox->noborder)
+			(*term->boxed_text)(x + htic, y + vtic, TEXTBOX_INIT);
+		    if (this_label->rotate && (*term->text_angle) (this_label->rotate)) {
+			write_multiline(x + htic, y + vtic, this_label->text,
+				    this_label->pos, justify, this_label->rotate,
+				    this_label->font);
+			(*term->text_angle) (0);
+		    } else {
+			write_multiline(x + htic, y + vtic, this_label->text,
+				    this_label->pos, justify, 0, this_label->font);
+		    }
+		}
 	    }
-	}
 
-	/* Draw the bounding box - FIXME should set line properties first */
-	if (this_label->boxed && term->boxed_text) {
-	    if (!textbox_opts.noborder)
+	    /* Draw the bounding box */
+	    if (!textbox->noborder) {
+		(*term->linewidth)(textbox->linewidth);
+		apply_pm3dcolor(&textbox->border_color);
 		(*term->boxed_text)(0,0, TEXTBOX_OUTLINE);
-	    else
-		(*term->boxed_text)(0,0, TEXTBOX_FINISH);
+	    }
+
+	    (*term->boxed_text)(0,0, TEXTBOX_FINISH);
 	}
-#endif
 
 	/* The associated point, if any */
 	/* write_multiline() clips text to on_page; do the same for any point */
 	if ((this_label->lp_properties.flags & LP_SHOW_POINTS) && on_page(x,y)) {
 	    term_apply_lp_properties(&this_label->lp_properties);
-	    (*term->point) (x, y, this_label->lp_properties.p_type);
+	    if (this_label->lp_properties.p_type == PT_CHARACTER) {
+		(*term->justify_text) (CENTRE);
+		(*term->put_text) (x, y, this_label->lp_properties.p_char);
+	    } else if (this_label->lp_properties.p_type >= 0) {
+		(*term->point) (x, y, this_label->lp_properties.p_type);
+	    }
 	    /* the default label color is that of border */
 	    term_apply_lp_properties(&border_lp);
 	}
@@ -844,7 +1074,7 @@ label_width(const char *str, int *lines)
     s = lab;
     while ((e = (char *) strchr(s, '\n')) != NULL) {
 	*e = '\0';
-	len = estimate_strlen(s);	/* = e-s ? */
+	len = estimate_strlen(s, NULL);	/* = e-s ? */
 	if (len > mlen)
 	    mlen = len;
 	if (len || l || *str == '\n')
@@ -859,3 +1089,187 @@ label_width(const char *str, int *lines)
     return (mlen);
 }
 
+/*
+ * Here so that it can be shared by the 2D and 3D code
+ */
+void
+do_timelabel(int x, int y)
+{
+    struct text_label temp = timelabel;
+    char str[MAX_LINE_LEN+1];
+    time_t now;
+
+    if (timelabel.rotate == 0 && !timelabel_bottom)
+	y -= term->v_char;
+
+    time(&now);
+    strftime(str, MAX_LINE_LEN, timelabel.text, localtime(&now));
+    temp.text = str;
+
+    write_label(x, y, &temp);
+}
+
+void
+init_gadgets()
+{
+   int i;
+   static t_position y0_wall_corners[5] = WALL_Y0_CORNERS;
+   static t_position x0_wall_corners[5] = WALL_X0_CORNERS;
+   static t_position y1_wall_corners[5] = WALL_Y1_CORNERS;
+   static t_position x1_wall_corners[5] = WALL_X1_CORNERS;
+   static t_position z0_wall_corners[5] = WALL_Z0_CORNERS;
+   for (i=0; i<4; i++)
+       grid_wall[i].next = &grid_wall[i+1];
+
+   grid_wall[WALL_Y0_TAG].o.polygon.vertex = y0_wall_corners;
+   grid_wall[WALL_X0_TAG].o.polygon.vertex = x0_wall_corners;
+   grid_wall[WALL_Y1_TAG].o.polygon.vertex = y1_wall_corners;
+   grid_wall[WALL_X1_TAG].o.polygon.vertex = x1_wall_corners;
+   grid_wall[WALL_Z0_TAG].o.polygon.vertex = z0_wall_corners;
+   grid_wall[WALL_Y0_TAG].lp_properties.pm3d_color.type = TC_RGB;
+   grid_wall[WALL_X0_TAG].lp_properties.pm3d_color.type = TC_RGB;
+   grid_wall[WALL_Y1_TAG].lp_properties.pm3d_color.type = TC_RGB;
+   grid_wall[WALL_X1_TAG].lp_properties.pm3d_color.type = TC_RGB;
+   grid_wall[WALL_Z0_TAG].lp_properties.pm3d_color.type = TC_RGB;
+   grid_wall[WALL_Y0_TAG].lp_properties.pm3d_color.lt = WALL_Y_COLOR;
+   grid_wall[WALL_X0_TAG].lp_properties.pm3d_color.lt = WALL_X_COLOR;
+   grid_wall[WALL_Y1_TAG].lp_properties.pm3d_color.lt = WALL_Y_COLOR;
+   grid_wall[WALL_X1_TAG].lp_properties.pm3d_color.lt = WALL_X_COLOR;
+   grid_wall[WALL_Z0_TAG].lp_properties.pm3d_color.lt = WALL_Z_COLOR;
+}
+
+/*
+ * walk through the list of objects to see if any require pm3d processing
+ */
+TBOOLEAN
+pm3d_objects(void)
+{
+    struct object *obj = first_object;
+    while (obj) {
+	if (obj->layer == LAYER_DEPTHORDER)
+	    return TRUE;
+	obj = obj->next;
+    }
+    return FALSE;
+}
+
+/*
+ * Place overall title on the canvas (shared by plot and splot).
+ */
+void
+place_title(int title_x, int title_y)
+{
+    if (title.text) {
+	/* NB: write_label applies text color but does not reset it */
+	write_label(title_x, title_y, &title);
+	reset_textcolor(&(title.textcolor));
+    }
+}
+
+/*
+ * Image pixels or pm3d quadrangles can be filtered by masking against
+ * a set of polygons that was stored in mask_XXpolygon_set by a previous
+ * plot/splot command with style POLYGONMASK ("with mask").
+ * These routines implement a test based on the Jordan curve theorem:
+ * If a line through a point intersects the bounding curve of a polygon
+ * an even number of times, the point is exterior. If the line intersects
+ * the bounding curve an odd number of times, the point is interior.
+ *
+ * interior(x,y,<polygon>) returns TRUE if the point is interior to
+ * the polygon defined by a list of vertices.
+ *
+ * masked(x,y,<polygon set>) returns TRUE if the point is *not*
+ * interior to any of the polygons in the set.
+ */
+static TBOOLEAN
+interior( double x, double y, struct coordinate *mask, int mask_vertices )
+{
+    int count = 0;
+    int i, j;
+
+    if (mask == NULL)
+	return FALSE;
+
+    for (i = 0, j = mask_vertices-1; i < mask_vertices; j = i++) {
+	if ( ((mask[i].y > y) != (mask[j].y > y))
+	&&   (x < (mask[j].x-mask[i].x) * (y-mask[i].y) / (mask[j].y-mask[i].y) + mask[i].x)
+	) {
+	    count++;
+	}
+    }
+
+    return (count & 0x1);
+}
+
+TBOOLEAN
+masked( double x, double y, struct iso_curve *mask_set )
+{
+    struct iso_curve *mask;
+
+    if (!mask_set)
+	return FALSE;
+    for (mask = mask_set; mask; mask = mask->next) {
+	if (interior(x, y, mask->points, mask->p_count))
+	    return FALSE;
+    }
+    return TRUE;
+}
+
+/*
+ * The 2D input stage places polygons in a single list of vertices
+ * with an UNDEFINED point acting as a separator between polygons.
+ * construct_2D_mask_set() creates a linked list of iso_curves,
+ * each pointing to one of the polygons in the original list.
+ * This allows both the 2D and 3D code to use the same masking tests.
+ */
+void
+construct_2D_mask_set( struct coordinate *points, int p_count )
+{
+    struct iso_curve *mask = mask_2Dpolygon_set;
+    struct coordinate *polygon;
+    struct coordinate *end_of_list;
+    int nv;
+
+    /* Free previous mask set, if any */
+    while (mask) {
+	struct iso_curve *temp = mask;
+	mask = mask->next;
+	free(temp);
+    }
+    mask_2Dpolygon_set = NULL;
+    if (points == NULL || p_count < 3)
+	return;
+
+    /* point type UNDEFINED separates multiple polygons */
+    end_of_list = &points[p_count];
+    polygon = &points[0];
+    nv = 0;
+    while (&polygon[nv] < end_of_list) {
+	for (nv = 1; &polygon[nv] < end_of_list; nv++) {
+	    if (polygon[nv].type == UNDEFINED)
+		break;
+	}
+	mask = gp_alloc( sizeof(struct iso_curve), "2D mask set");
+	mask->next = mask_2Dpolygon_set;
+        mask->points = polygon;
+	mask->p_count = nv;
+	mask->p_max = 0;
+	mask_2Dpolygon_set = mask;
+	polygon = &polygon[nv];
+	nv = 0;
+	if (polygon < end_of_list)
+	    polygon++;
+    }
+}
+
+/* Calculate the region on the canvas used by the current plot.
+ * This can be saved to guide subsequent mousing.
+ */
+void
+update_active_region(void)
+{
+    active_bounds.xleft = term->xmax * xoffset;
+    active_bounds.xright = term->xmax * (xoffset + xsize);
+    active_bounds.ybot = term->ymax * yoffset;
+    active_bounds.ytop = term->ymax * (yoffset + ysize);
+}
