@@ -1,7 +1,3 @@
-/*
- * $Id: gp_types.h,v 1.63.2.1 2014/12/05 18:09:34 sfeam Exp $
- */
-
 /* GNUPLOT - gp_types.h */
 
 /*[
@@ -50,8 +46,17 @@ enum DATA_TYPES {
 	CMPLX,
 	STRING,
 	DATABLOCK,
+	FUNCTIONBLOCK,
+	ARRAY,
+			/* ARRAY subcategories are marked in array->value_array[0].type */
+	COLORMAP_ARRAY,	/*    Array containing packed ARGB values */
+	TEMP_ARRAY,	/*    Array with lifetime limited to evaluation stack */
+	LOCAL_ARRAY,	/*    Array declared local (function block or bracketed clause) */
+	VOXELGRID,
+	NOTDEFINED,	/* exists, but value is currently undefined */
 	INVALID_VALUE,	/* used only for error return by external functions */
 	INVALID_NAME	/* used only to trap errors in linked axis function definition */
+			/* or a format specifier that does not match a variable type */
 };
 
 enum MODE_PLOT_TYPE {
@@ -59,7 +64,7 @@ enum MODE_PLOT_TYPE {
 };
 
 enum PLOT_TYPE {
-	FUNC, DATA, FUNC3D, DATA3D, NODATA
+	FUNC, DATA, FUNC3D, DATA3D, NODATA, KEYENTRY, VOXELDATA
 };
 
 /* we explicitly assign values to the types, such that we can
@@ -75,7 +80,9 @@ typedef enum e_PLOT_STYLE_FLAGS {
     PLOT_STYLE_HAS_POINT     = (1<<1),
     PLOT_STYLE_HAS_ERRORBAR  = (1<<2),
     PLOT_STYLE_HAS_FILL      = (1<<3),
-    PLOT_STYLE_BITS          = (1<<4)
+    PLOT_STYLE_HAS_VECTOR    = (1<<4),
+    PLOT_STYLE_HAS_PM3DBORDER= (1<<5),
+    PLOT_STYLE_BITS          = (1<<6)
 } PLOT_STYLE_FLAGS;
 
 typedef enum PLOT_STYLE {
@@ -88,20 +95,20 @@ typedef enum PLOT_STYLE {
     YERRORBARS   =  6*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_POINT | PLOT_STYLE_HAS_ERRORBAR),
     XYERRORBARS  =  7*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_POINT | PLOT_STYLE_HAS_ERRORBAR),
     BOXXYERROR   =  8*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_FILL),
-    BOXES        =  9*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_FILL),
+    BOXES        =  9*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_FILL | PLOT_STYLE_HAS_PM3DBORDER),
     BOXERROR     = 10*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_FILL),
     STEPS        = 11*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
     FILLSTEPS    = 11*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
     FSTEPS       = 12*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
     HISTEPS      = 13*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
-    VECTOR       = 14*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
+    VECTOR       = 14*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_VECTOR,
     CANDLESTICKS = 15*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_ERRORBAR | PLOT_STYLE_HAS_FILL),
     FINANCEBARS  = 16*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
     XERRORLINES  = 17*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_POINT | PLOT_STYLE_HAS_ERRORBAR),
     YERRORLINES  = 18*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_POINT | PLOT_STYLE_HAS_ERRORBAR),
     XYERRORLINES = 19*PLOT_STYLE_BITS + (PLOT_STYLE_HAS_LINE | PLOT_STYLE_HAS_POINT | PLOT_STYLE_HAS_ERRORBAR),
     FILLEDCURVES = 21*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_FILL,
-    PM3DSURFACE  = 22*PLOT_STYLE_BITS + 0,
+    PM3DSURFACE  = 22*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
     LABELPOINTS  = 23*PLOT_STYLE_BITS + 0,
     HISTOGRAMS   = 24*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
     IMAGE        = 25*PLOT_STYLE_BITS + 0,
@@ -110,14 +117,22 @@ typedef enum PLOT_STYLE {
     CIRCLES      = 28*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_FILL,
     BOXPLOT      = 29*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL + PLOT_STYLE_HAS_POINT,
     ELLIPSES     = 30*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_FILL,
-    SURFACEGRID  = 31*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
+    SURFACEGRID  = 31*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_FILL,
     PARALLELPLOT = 32*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE,
     TABLESTYLE   = 33*PLOT_STYLE_BITS,
+    ZERRORFILL   = 34*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
+    ARROWS       = 35*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_VECTOR,
+    ISOSURFACE   = 36*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL + PLOT_STYLE_HAS_PM3DBORDER,
+    SPIDERPLOT   = 37*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL + PLOT_STYLE_HAS_POINT,
+    POLYGONS     = 38*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
+    POLYGONMASK  = 39*PLOT_STYLE_BITS,
+    SECTORS      = 40*PLOT_STYLE_BITS + PLOT_STYLE_HAS_LINE + PLOT_STYLE_HAS_FILL,
+    CONTOURFILL  = 41*PLOT_STYLE_BITS + PLOT_STYLE_HAS_FILL,
     PLOT_STYLE_NONE = -1
 } PLOT_STYLE;
 
 typedef enum PLOT_SMOOTH {
-    SMOOTH_NONE,
+    SMOOTH_NONE = 0,
     SMOOTH_ACSPLINES,
     SMOOTH_BEZIER,
     SMOOTH_CSPLINES,
@@ -128,24 +143,44 @@ typedef enum PLOT_SMOOTH {
     SMOOTH_CUMULATIVE,
     SMOOTH_KDENSITY,
     SMOOTH_CUMULATIVE_NORMALISED,
-    SMOOTH_MONOTONE_CSPLINE
+    SMOOTH_MONOTONE_CSPLINE,
+    SMOOTH_BINS,
+    SMOOTH_FREQUENCY_NORMALISED,
+    SMOOTH_ZSORT,
+    SMOOTH_PATH,
+    SMOOTH_SMOOTH_HULL	/* DEPRECATED */
 } PLOT_SMOOTH;
 
-/* FIXME HBB 20000521: 'struct value' and its part, 'cmplx', should go
- * into one of scanner/internal/standard/util .h, but I've yet to
- * decide which of them */
+typedef enum PLOT_FILTER {
+    FILTER_NONE = 0,
+    FILTER_BINS,
+    FILTER_CONVEX_HULL,
+    FILTER_CONCAVE_HULL,
+    FILTER_DELAUNAY,
+    FILTER_MASK,
+    FILTER_ZSORT,
+    FILTER_SHARPEN
+} PLOT_FILTER;
 
 struct cmplx {
 	double real, imag;
 };
 
+struct fblock {
+	char **data_array;
+	char **parnames;
+};
+
 typedef struct value {
     enum DATA_TYPES type;
     union {
-	int int_val;
+	intgr_t int_val;
 	struct cmplx cmplx_val;
 	char *string_val;
 	char **data_array;
+	struct fblock functionblock;
+	struct value *value_array;
+	struct vgrid *vgrid;
     } v;
 } t_value;
 
@@ -160,25 +195,52 @@ typedef enum coord_type {
 } coord_type;
 
 
-/* These fields of 'struct coordinate' used for storing the color of 3D data
- * points (if requested by NEED_PALETTE(this_plot), for instance).
+/* These are aliases of fields in 'struct coordinate' used to hold
+ * extra properties of 3D data points (i.e. anything other than x/y/z)
+ * or of specific plot types.
+ * The aliases are needed because the total number of data slots is limited
+ * to 7: x y z xlow ylow xhigh yhigh
+ * At some point we may need to expand struct coordinate.
  */
-#define CRD_COLOR yhigh
-#define CRD_R yhigh
-#define CRD_G xlow
-#define CRD_B xhigh
-#define CRD_A ylow
-/* The field of 'struct coordinate' used for storing the point size in plot
- * style POINTSTYLE with variable point size
- */
-#define CRD_PTSIZE xlow
+#define CRD_R yhigh        /* Used by splot styles RGBIMAGE and RGBA_IMAGE */
+#define CRD_G xlow         /* Used by splot styles RGBIMAGE and RGBA_IMAGE */
+#define CRD_B xhigh        /* Used by splot styles RGBIMAGE and RGBA_IMAGE */
+#define CRD_A ylow         /* Used by splot styles RGBIMAGE and RGBA_IMAGE */
+#define CRD_ASTYLE ylow    /* Used by splot styles VECTOR and ARROWS */
+#define CRD_ZLOW xlow      /* Used by splot style ZERRORFILL */
+#define CRD_ZHIGH xhigh    /* Used by splot style ZERRORFILL */
+#define CRD_COLOR yhigh    /* Used by all splot styles with variable color */
 
+#define CRD_ROTATE ylow    /* Used by "with labels" */
+#define CRD_PTSIZE xlow    /* Used by "with points|linespoints|labels" */
+#define CRD_PTTYPE xhigh   /* Used by "with points|linespoints|labels" */
+#define CRD_PTCHAR ylow    /* Used by "with points pt variable */
+#define CRD_XJITTER xlow   /* Used to hold jitter offset on x */
+#define CRD_YJITTER yhigh  /* Used to hold jitter offset on y */
+#define CRD_PATH xhigh     /* Used by 3D spline code to hold path coordinate */
+#define PATHCOORD 6        /*    must match sequence order of field CRD_PATH */
+
+/*
+ * If the C compiler pads struct coordinate to an 8-byte boundary
+ * (true for all the compilers I have tested) then there is a hole
+ * that we might as well use to hold an extra coordinate property.
+ * The Delaunay triangulation code needs this extra storage.
+ */
+#if defined(WITH_CHI_SHAPES) && !(defined(WITH_EXTRA_COORDINATE))
+  #define WITH_EXTRA_COORDINATE
+#endif
+#ifdef WITH_EXTRA_COORDINATE
+  #define EXTRA_COORDINATE int extra;
+#else
+  #define EXTRA_COORDINATE
+#endif
 
 typedef struct coordinate {
-    enum coord_type type;	/* see above */
     coordval x, y, z;
     coordval ylow, yhigh;	/* ignored in 3d */
     coordval xlow, xhigh;	/* also ignored in 3d */
+    enum coord_type type;	/* INRANGE/OUTRANGE/UNDEFINED */
+    EXTRA_COORDINATE		/* otherwise unused space in structure */
 } coordinate;
 
 typedef enum lp_class {
@@ -188,4 +250,21 @@ typedef enum lp_class {
 	LP_NOFILL = 3	/* special treatment of fillcolor */
 } lp_class;
 
+/* Classes of time data */
+typedef enum {
+    DT_NORMAL=0,		/* default; treat values as pure numeric */
+    DT_TIMEDATE,		/* old datatype */
+    DT_DMS,			/* degrees minutes seconds */
+    DT_UNINITIALIZED,
+    DT_BAD			/* something went wrong (e.g. in gstrptime) */
+} td_type;
+
+/*
+ * Introduction of nonlinear axes makes it possible for an axis-mapping function
+ * to return "undefined" or NaN. These cannot be encoded as an integer coordinate.
+ * So we introduce an integer equivalent to NaN and provide a macro to test for
+ * whether a coordinate mapping returned it.
+ */
+#define intNaN (~((unsigned int)(~0)>>1))
+
 #endif /* GNUPLOT_GPTYPES_H */
