@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: history.c,v 1.41 2016/07/21 05:50:38 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - history.c */
 
 /*[
@@ -92,10 +88,10 @@ write_history(char *filename)
 
 /* routine to read history entries from a file
  */
-void
+int
 read_history(char *filename)
 {
-    gp_read_history(filename);
+    return gp_read_history(filename);
 }
 
 
@@ -351,6 +347,7 @@ history_search_prefix(const char *string, int direction)
 }
 #endif
 
+#ifdef GNUPLOT_HISTORY
 
 /* routine to read history entries from a file,
  * this complements write_history and is necessary for
@@ -390,6 +387,7 @@ gp_read_history(const char *filename)
 	return errno;
     }
 }
+#endif
 
 
 #ifdef USE_READLINE
@@ -484,16 +482,6 @@ history_find_by_number(int n)
 const char *
 history_find(char *cmd)
 {
-    int len;
-
-    /* remove quotes */
-    if (*cmd == '"')
-	cmd++;
-    if (!*cmd)
-	return NULL;
-    len = strlen(cmd);
-    if (cmd[len - 1] == '"')
-	cmd[--len] = NUL;
     if (!*cmd)
 	return NULL;
 
@@ -505,7 +493,6 @@ history_find(char *cmd)
 #endif
 
     /* Anchored backward search for prefix */
-    /* FIXME: the built-in readline version used to ignore leading spaces */
     if (history_search_prefix(cmd, -1) == 0)
 	return current_history()->line;
     return NULL;
@@ -522,19 +509,10 @@ history_find(char *cmd)
 int
 history_find_all(char *cmd)
 {
-    int len;
     int found;
     int ret;
     int number = 0; /* each entry found increases this */
 
-    /* remove quotes */
-    if (*cmd == '"')
-	cmd++;
-    if (!*cmd)
-	return 0;
-    len = strlen(cmd);
-    if (cmd[len - 1] == '"')
-	cmd[--len] = 0;
     if (!*cmd)
 	return 0;
 
