@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: multiplot.c,v 1.6 2016/09/14 18:37:38 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - term.c */
 
 /*[
@@ -45,15 +41,26 @@ static char *RCSid() { return RCSid("$Id: multiplot.c,v 1.6 2016/09/14 18:37:38
 #include "multiplot.h"
 
 #include "command.h"
+#include "datablock.h"
 #include "gadgets.h"
 #include "graphics.h"
+#include "misc.h"
 #include "parse.h"
+#include "setshow.h"
 #include "util.h"
 
+/* Local prototypes */
+static void mp_layout_size_and_offset(void);
+static void mp_layout_margins_and_spacing(void);
+static void mp_layout_set_margin_or_spacing(t_position *);
+static void init_multiplot_datablock(void);
 
-static void mp_layout_size_and_offset __PROTO((void));
-static void mp_layout_margins_and_spacing __PROTO((void));
-static void mp_layout_set_margin_or_spacing __PROTO((t_position *));
+/* Support for multiplot playback */
+TBOOLEAN multiplot_playback = FALSE;	/* TRUE while inside "remultiplot" playback */
+static t_value multiplot_udv = {
+	.type = DATABLOCK,
+	.v.data_array = NULL
+};
 
 enum set_multiplot_id {
     S_MULTIPLOT_LAYOUT,
@@ -122,7 +129,7 @@ static struct {
     double prev_xsize, prev_ysize, prev_xoffset, prev_yoffset;
     t_position prev_lmargin, prev_rmargin, prev_tmargin, prev_bmargin;
 			   /* values before 'set multiplot layout' */
-    text_label title;    /* goes above complete set of plots */
+    text_label title;      /* goes above complete set of plots */
     double title_height;   /* fractional height reserved for title */
 } mp_layout = MP_LAYOUT_DEFAULT;
 
@@ -153,15 +160,12 @@ multiplot_next()
 		}
 	    }
 	}
-	if (mp_layout.auto_layout_margins)
-	    mp_layout_margins_and_spacing();
-	else
-	    mp_layout_size_and_offset();
+	multiplot_reset();
     }
 }
 
 void
-multiplot_previous()
+multiplot_previous(void)
 {
     mp_layout.current_panel--;
     if (mp_layout.auto_layout) {
@@ -186,10 +190,7 @@ multiplot_previous()
 		}
 	    }
 	}
-	if (mp_layout.auto_layout_margins)
-	    mp_layout_margins_and_spacing();
-	else
-	    mp_layout_size_and_offset();
+	multiplot_reset();
     }
 }
 
@@ -208,7 +209,6 @@ multiplot_start()
     c_token++;
 
     /* Only a few options are possible if we are already in multiplot mode */
-    /* So far we have "next".  Maybe also "previous", "clear"? */
     if (multiplot) {
 	if (equals(c_token, "next")) {
 	    c_token++;
@@ -243,34 +243,13 @@ multiplot_start()
 
 	if (almost_equals(c_token, "ti$tle")) {
 	    c_token++;
-	    mp_layout.title.text = try_to_get_string();
+	    parse_label_options(&mp_layout.title, 2);
+	    if (!END_OF_COMMAND)
+		mp_layout.title.text = try_to_get_string();
+	    parse_label_options(&mp_layout.title, 2);
  	    continue;
-       }
-
-       if (equals(c_token, "font")) {
-	    c_token++;
-	    mp_layout.title.font = try_to_get_string();
-	    continue;
 	}
 
-        if (almost_equals(c_token,"enh$anced")) {
-            mp_layout.title.noenhanced = FALSE;
-            c_token++;
-            continue;
-        }
-
-        if (almost_equals(c_token,"noenh$anced")) {
-            mp_layout.title.noenhanced = TRUE;
-            c_token++;
-            continue;
-        }
-        
-	if (equals(c_token,"boxed")) {
-            mp_layout.title.boxed = 1;
-            c_token++;
-            continue;
-        }
-
 	if (almost_equals(c_token, "lay$out")) {
 	    if (mp_layout.auto_layout)
 		int_error(c_token, "too many layout commands");
@@ -278,9 +257,8 @@ multiplot_start()
 		mp_layout.auto_layout = TRUE;
 
 	    c_token++;
-	    if (END_OF_COMMAND) {
+	    if (END_OF_COMMAND)
 		int_error(c_token,"expecting '<num_cols>,<num_rows>'");
-	    }
 
 	    /* read row,col */
 	    mp_layout.num_rows = int_expression();
@@ -388,7 +366,7 @@ multiplot_start()
 		    mp_layout.tmargin.scalex = mp_layout.bmargin.scalex;
 		    mp_layout_set_margin_or_spacing(&(mp_layout.tmargin));
 		} else {
-		    int_error(c_token, "expection <top>");
+		    int_error(c_token, "expecting <top>");
 		}
 		set_margins = TRUE;
 		break;
@@ -445,7 +423,9 @@ multiplot_start()
      */
     term_start_plot();
     multiplot = TRUE;
+    multiplot_count = 0;
     fill_gpval_integer("GPVAL_MULTIPLOT", 1);
+    init_multiplot_datablock();
 
     /* Place overall title before doing anything else */
     if (mp_layout.title.text) {
@@ -477,16 +457,14 @@ multiplot_start()
 	mp_layout.title_height = 0.0;
     }
 
-    if (mp_layout.auto_layout_margins)
-	mp_layout_margins_and_spacing();
-    else
-	mp_layout_size_and_offset();
+    multiplot_reset();
 }
 
 void
 multiplot_end()
 {
     multiplot = FALSE;
+    multiplot_count = 0;
     fill_gpval_integer("GPVAL_MULTIPLOT", 0);
     /* reset plot size, origin and margins to values before 'set
        multiplot layout' */
@@ -516,9 +494,32 @@ multiplot_end()
 	free(mp_layout.title.text);
 	mp_layout.title.text = NULL;
     }
+
+    if (!multiplot_playback) {
+	/* Create or recycle a user-visible datablock for replay */
+	struct udvt_entry *datablock = add_udv_by_name("$GPVAL_LAST_MULTIPLOT");
+	free_value(&datablock->udv_value);
+
+	/* Save the current "unset multiplot" command before moving the command list */
+	append_to_datablock(&multiplot_udv, strdup(gp_input_line));
+
+	/* Move the command list to $GPVAL_LAST_MULTIPLOT */
+	datablock->udv_value = multiplot_udv;
+	multiplot_udv.v.data_array = NULL;
+    }
+    last_plot_was_multiplot = TRUE;
 }
 
 /* Helper function for multiplot auto layout to issue size and offset cmds */
+void
+multiplot_reset()
+{
+    if (mp_layout.auto_layout_margins)
+	mp_layout_margins_and_spacing();
+    else
+	mp_layout_size_and_offset();
+}
+
 static void
 mp_layout_size_and_offset(void)
 {
@@ -642,3 +643,49 @@ mp_layout_set_margin_or_spacing(t_position *margin)
     }
 }
 
+/* Begin accummulating lines that can later be replayed by replay_multiplot().
+ * The first line to be stored is a comment.
+ * Subsequent lines will be appended as they are read in.
+ */
+static void
+init_multiplot_datablock()
+{
+    gpfree_datablock(&multiplot_udv);
+    multiplot_udv.type = DATABLOCK;
+    append_to_datablock(&multiplot_udv, strdup("# saved multiplot"));
+}
+
+/* Append one line to the multiplot history.
+ * Called from two places:
+ *	command.c:com_line() catches direct input from stdin
+ *	misc.c:load_file() catches lines from load/call
+ * When the multiplot is exited via multiplot_end(), all lines will be
+ * copied to the user-visible datblock $GPVAL_LAST_MULTIPLOT.
+ */
+void
+append_multiplot_line(char *line)
+{
+    append_to_datablock(&multiplot_udv, strdup(line));
+}
+
+/* This is the implementation of "remultiplot".
+ * The load operation is protected by a flag to prevent infinite recursion.
+ */
+void
+replay_multiplot()
+{
+    c_token++;
+    multiplot_playback = TRUE;
+    load_file(NULL, strdup("$GPVAL_LAST_MULTIPLOT"), 6);
+    multiplot_playback = FALSE;
+}
+
+/* reset needed if int_error() is invoked during multiplot playback */
+void
+multiplot_reset_after_error()
+{
+    if (!multiplot_playback)
+	return;
+    multiplot_end();
+    multiplot_playback = FALSE;
+}
