@@ -1,7 +1,3 @@
-/*
- * $Id: pm3d.h,v 1.31 2014/04/02 21:35:46 sfeam Exp $
- */
-
 /* GNUPLOT - pm3d.h */
 
 /*[
@@ -26,7 +22,8 @@
 #ifndef PM3D_H
 #define PM3D_H
 
-#include "graph3d.h" /* struct surface_points */
+#include "axis.h"	/* only for NONLINEAR_AXES */
+#include "graph3d.h"	/* struct surface_points */
 
 
 
@@ -37,7 +34,7 @@
 /*
   where to plot pm3d: base or top (color map) or surface (color surface)
   The string pm3d.where can be any combination of the #defines below.
-  For instance, "b" plot at botton only, "st" plots firstly surface, then top, etc.
+  For instance, "b" plot at bottom only, "st" plots firstly surface, then top, etc.
 */
 #define PM3D_AT_BASE	'b'
 #define PM3D_AT_TOP	't'
@@ -56,17 +53,21 @@
   direction of taking the scans: forward = as the scans are stored in the
   file; backward = opposite direction, i.e. like from the end of the file
 */
-#define PM3D_SCANS_AUTOMATIC  'a'
-#define PM3D_SCANS_FORWARD    'f'
-#define PM3D_SCANS_BACKWARD   'b'
-#define PM3D_DEPTH            'd'
+typedef enum {
+    PM3D_SCANS_AUTOMATIC,
+    PM3D_SCANS_FORWARD,
+    PM3D_SCANS_BACKWARD,
+    PM3D_DEPTH
+} pm3d_scandir;
 
 /*
   clipping method:
     PM3D_CLIP_1IN: all 4 points of the quadrangle must be defined and at least
 		   1 point of the quadrangle must be in the x and y ranges
     PM3D_CLIP_4IN: all 4 points of the quadrangle must be in the x and y ranges
+    PM3D_CLIP_Z:   smooth clip to current zrange
 */
+#define PM3D_CLIP_Z 0
 #define PM3D_CLIP_1IN '1'
 #define PM3D_CLIP_4IN '4'
 
@@ -92,11 +93,13 @@ typedef enum {
     PM3D_WHICHCORNER_GEOMEAN = 5, /* geometrical mean of 4 corners */
     PM3D_WHICHCORNER_HARMEAN = 6, /* harmonic mean of 4 corners */
     PM3D_WHICHCORNER_MEDIAN  = 7, /* median of 4 corners */
-    PM3D_WHICHCORNER_RMS	 = 8,  /* root mean square of 4 corners*/
+    PM3D_WHICHCORNER_RMS     = 8, /* root mean square of 4 corners*/
     PM3D_WHICHCORNER_MIN     = 9, /* minimum of 4 corners */
-    PM3D_WHICHCORNER_MAX     = 10,  /* maximum of 4 corners */
+    PM3D_WHICHCORNER_MAX     = 10,/* maximum of 4 corners */
+    PM3D_COLOR_BY_NORMAL     = 11 /* derive color from surface normal (not currently used) */
 } PM3D_WHICH_CORNERS2COLOR;
 
+
 /*
   structure defining all properties of pm3d plotting mode
   (except for the properties of the smooth color box, see color_box instead)
@@ -105,8 +108,12 @@ typedef struct {
   char where[7];	/* base, top, surface */
   char flush;   	/* left, right, center */
   char ftriangles;   	/* 0/1 (don't) draw flushing triangles */
-  char direction;	/* forward, backward */
   char clip;		/* 1in, 4in */
+  TBOOLEAN no_clipcb;	/* FALSE: out-of-range cb treated as 0  TRUE: treated as NaN */
+  pm3d_scandir direction;
+			/* If direction is "depth" sort by max z of 4 corners unless ... */
+  TBOOLEAN base_sort;	/*     use z values of projection to baseplane */
+  TBOOLEAN zmean_sort;	/*     sort on mean z rather than max z */
   PM3D_IMPL_MODE implicit;
 			/* 1: [default] draw ALL surfaces with pm3d
 			   0: only surfaces specified with 'with pm3d' */
@@ -119,34 +126,59 @@ typedef struct {
 
 extern pm3d_struct pm3d;
 
+typedef struct lighting_model {
+  double strength;	/* 0 = no lighting model; 1 = full shading */
+  double spec;		/* specular component 0-1 */
+  double ambient;	/* ambient component 0-1 */
+  double Phong;		/* Phong exponent */
+  int rot_z;		/* illumination angle */
+  int rot_x;		/* illumination angle */
+  TBOOLEAN fixed;	/* TRUE means the light does not rotate */
+  double spec2;		/* 2nd specular contribution from red spotlight on opposite side */
+  double spec2_rot_x;
+  double spec2_rot_z;
+  double spec2_Phong;
+  unsigned long spec2_rgb;
+} lighting_model;
+
+extern lighting_model pm3d_shade;
+
+typedef struct zslice {
+  double zlow;
+  double zhigh;
+  t_colorspec color;
+} zslice;
+
 /* Used to initialize `set pm3d border` */
 extern struct lp_style_type default_pm3d_border;
 
-/* Used by routine filled_quadrangle() in color.c */
-extern struct lp_style_type pm3d_border_lp;	/* FIXME: Needed anymore? */
-
+/* Set by plot styles that use pm3d quadrangles even in non-pm3d mode */
+extern TBOOLEAN track_pm3d_quadrangles;
 
 
 /****
   Declaration of routines
 ****/
 
-int get_pm3d_at_option __PROTO((char *pm3d_where));
-void pm3d_depth_queue_clear __PROTO((void));
-void pm3d_depth_queue_flush __PROTO((void));
-void pm3d_reset __PROTO((void));
-void pm3d_draw_one __PROTO((struct surface_points* plots));
-double z2cb __PROTO((double z));
-double cb2gray __PROTO((double cb));
-void
-pm3d_rearrange_scan_array __PROTO((struct surface_points* this_plot,
+int get_pm3d_at_option(char *pm3d_where);
+void pm3d_depth_queue_clear(void);
+void pm3d_depth_queue_flush(void);
+void pm3d_reset(void);
+void pm3d_draw_one(struct surface_points* plots);
+void pm3d_add_quadrangle(struct surface_points* plot, gpdPoint *corners);
+void pm3d_add_polygon(struct surface_points* plot, gpdPoint *corners, int vertices);
+void pm3d_init_lighting_model(void);
+int pm3d_side( struct coordinate *p0, struct coordinate *p1, struct coordinate *p2);
+double cb2gray(double cb);
+void pm3d_rearrange_scan_array(struct surface_points* this_plot,
     struct iso_curve*** first_ptr, int* first_n, int* first_invert,
-    struct iso_curve*** second_ptr, int* second_n, int* second_invert));
+    struct iso_curve*** second_ptr, int* second_n, int* second_invert);
 
-void set_plot_with_palette __PROTO((int plot_num, int plot_mode));
+void set_plot_with_palette(int plot_num, int plot_mode);
+void pm3d_reset_after_error(void);
 
-TBOOLEAN is_plot_with_palette __PROTO((void));
-TBOOLEAN is_plot_with_colorbox __PROTO((void));
+TBOOLEAN is_plot_with_palette(void);
+TBOOLEAN is_plot_with_colorbox(void);
 
 #endif /* PM3D_H */
 
