@@ -62,12 +62,13 @@ extern "C" {
 	#include "alloc.h"     // for gp_alloc
 	#include "parse.h"     // for real_expression
 	#include "axis.h"
-#ifdef WIN32
-	#include "win/winmain.h"  // for WinMessageLoop, ConsoleReadCh, stdin_pipe_reader
+#ifdef _WIN32
+	#include "win/winmain.h"  // for WinMessageLoop, ConsoleReadCh, init_pipe_input
 	#include "win/wgnuplib.h" // for TextStartEditing, TextStopEditing
 	#include "win/wtext.h"    // for kbhit, getchar
 	#include <io.h>           // for isatty
 #endif
+	#include "readline.h"	// for wrap_readline_signal_handler() to catch ^Z
 	#include <signal.h>
 }
 
@@ -94,7 +95,7 @@ struct QtGnuplotState {
      *-------------------------------------------------------*/
 
     bool gnuplot_qtStarted;
-    int  currentFontSize;
+    double currentFontSize;
     QString currentFontName;
     QString localServerName;
     QTextCodec* codec;
@@ -105,6 +106,9 @@ struct QtGnuplotState {
     QLocalSocket socket;
     QByteArray   outBuffer;
     QDataStream  out;
+#ifdef _WIN32
+    DWORD        pid;
+#endif
 
     bool       enhancedSymbol;
     QString    enhancedFontName;
@@ -159,11 +163,20 @@ static bool qt_optionEnhanced = true;
 static bool qt_optionPersist  = false;
 static bool qt_optionRaise    = true;
 static bool qt_optionCtrl     = false;
+static bool qt_optionRounded  = true; 
+static bool qt_optionAntiAlias  = true;
+static bool qt_optionReplotOnResize  = true;
 static bool qt_optionDash     = true;
 static int  qt_optionWidth    = 640;
 static int  qt_optionHeight   = 480;
-static int  qt_optionFontSize = 9;
+static double qt_optionFontSize = 10.0;
 static double qt_optionDashLength = 1.0;
+static double qt_optionLineWidth = 1.0;
+
+static int qt_optionctrlq = -1;	         // tristate -1 = not set 0 = false 1 = true 
+static int qt_optionrounded = -1;	 // tristate -1 = not set 0 = false 1 = true 
+static int qt_optionantialias = -1;	 // tristate -1 = not set 0 = false 1 = true 
+static int qt_optionreplotonresize = -1; // tristate -1 = not set 0 = false 1 = true 
 
 /* Encapsulates all Qt options that have a constructor and destructor. */
 struct QtOption {
@@ -174,45 +187,56 @@ struct QtOption {
     QString FontName;
     QString Title;
     QString Widget;
-	QPoint  position;
+    QPoint  position;
 };
 static QtOption* qt_option = NULL;
 
 static void ensureOptionsCreated()
 {
 	if (!qt_option)
-	    qt_option = new QtOption();
+		qt_option = new QtOption();
 }
 
 static bool qt_setPosition = false;
 static bool qt_setSize   = true;
 static int  qt_setWidth  = qt_optionWidth;
 static int  qt_setHeight = qt_optionHeight;
+static bool qt_is_3Dplot = false;
+
+static double qt_max_pos_base = 0.0;
+static double qt_max_neg_base = 0.0;
 
 /* ------------------------------------------------------
  * Helpers
  * ------------------------------------------------------*/
+// Adjust for the mismatch of real coords 0->ymax and pixel coords [0:ymax-1]
+#ifndef QT_YBASE
+#define QT_YBASE 1
+#endif
 
 // Convert gnuplot coordinates into floating point term coordinates
 QPointF qt_termCoordF(int x, int y)
 {
-	return QPointF(double(x)/qt_oversamplingF, double(int(term->ymax) - y)/qt_oversamplingF);
+	return QPointF(double(x)/qt_oversamplingF,
+			double(int(term->ymax) - QT_YBASE - y)/qt_oversamplingF);
 }
 
 // The same, but with coordinates clipped to the nearest pixel
 QPoint qt_termCoord(int x, int y)
 {
-	return QPoint(qRound(double(x)/qt_oversamplingF), qRound(double(term->ymax - y)/qt_oversamplingF));
+	return QPoint(qRound(double(x)/qt_oversamplingF),
+			qRound(double(term->ymax - QT_YBASE - y)/qt_oversamplingF));
 }
 
 // Inverse of the previous function
 QPoint qt_gnuplotCoord(int x, int y)
 {
-	return QPoint(x*qt_oversampling, int(term->ymax) - y*qt_oversampling);
+	return QPoint(x*qt_oversampling,
+			int(term->ymax) - QT_YBASE - y*qt_oversampling);
 }
 
 #ifndef GNUPLOT_QT
-# ifdef WIN32
+# ifdef _WIN32
 #  define GNUPLOT_QT "gnuplot_qt.exe"
 # else
 #  define GNUPLOT_QT "gnuplot_qt"
@@ -227,7 +251,7 @@ void execGnuplotQt()
 	if (path)
 		filename = QString(path);
 	if (filename.isEmpty()) {
-#ifdef WIN32
+#ifdef _WIN32
 		filename = QCoreApplication::applicationDirPath();
 #else
 		filename = QT_DRIVER_DIR;
@@ -239,10 +263,16 @@ void execGnuplotQt()
 
 	qint64 pid;
 	qt->gnuplot_qtStarted = QProcess::startDetached(filename, QStringList(), QString(), &pid);
-	if (qt->gnuplot_qtStarted)
+	if (qt->gnuplot_qtStarted) {
 		qt->localServerName = "qtgnuplot" + QString::number(pid);
-	else
+#ifdef _WIN32
+		// save the terminal's PID for later use
+		qt->pid = pid;
+#endif
+	} else {
 		fprintf(stderr, "Could not start gnuplot_qt with path %s\n", filename.toUtf8().data());
+		fprintf(stderr, "Did you set environmental variable GNUPLOT_DRIVER_DIR?\n");
+	}
 }
 
 /*-------------------------------------------------------
@@ -254,7 +284,7 @@ void qt_flushOutBuffer()
 	if (!qt || !qt->socket.isValid())
 		return;
 
-	// Write the block size at the beginning of the bock
+	// Write the block size at the beginning of the block
 	QDataStream sizeStream(&qt->socket);
 	sizeStream << (quint32)(qt->outBuffer.size());
 	// Write the block to the QLocalSocket
@@ -374,6 +404,19 @@ bool qt_processTermEvent(gp_event_t* event)
 		event->my = p.y();
 	}
 
+#ifdef _WIN32
+	if (event->type == GE_raise)
+	{
+# ifndef WGP_CONSOLE
+		SetForegroundWindow(textwin.hWndParent);
+# else
+		SetForegroundWindow(GetConsoleWindow());
+# endif
+		WinRaiseConsole();
+		return true;
+	}
+#endif
+
 	// Send the event to gnuplot core
 	do_event(event);
 	// Process pause_for_mouse
@@ -392,6 +435,11 @@ bool qt_processTermEvent(gp_event_t* event)
 		paused_for_mouse = 0;
 		return true;
 	}
+	if (event->type == GE_reset)
+	{
+		paused_for_mouse = 0;
+		return true;
+	}
 
 	return false;
 }
@@ -416,7 +464,7 @@ void qt_init()
 	// The creation of a QApplication mangled our locale settings
 #ifdef HAVE_LOCALE_H
 	setlocale(LC_NUMERIC, "C");
-	setlocale(LC_TIME, current_locale);
+	setlocale(LC_TIME, time_locale);
 #endif
 
 	qt->out.setVersion(QDataStream::Qt_4_4);
@@ -429,14 +477,14 @@ void qt_sendFont()
 {
 	qt->out << GESetFont << qt->currentFontName << qt->currentFontSize;
 
-	QPair<QString, int> currentFont(qt->currentFontName, qt->currentFontSize);
-	static QPair<QString, int> lastFont("", 0);
+	QPair<QString, double> currentFont(qt->currentFontName, qt->currentFontSize);
+	static QPair<QString, double> lastFont("", 0.0);
 
 	// The font has not changed
 	if (currentFont == lastFont)
 		return;
 
-	static QMap<QPair<QString, int>, QPair<int, int> > fontMetricCache;
+	static QMap< QPair<QString, double>, QPair<int, int> > fontMetricCache;
 	QPair<int, int> metric;
 
 	// Try to find the font metric in the cache or ask the GUI for the font metrics
@@ -454,13 +502,16 @@ void qt_sendFont()
 			if (qt->socket.bytesAvailable() < (int)sizeof(gp_event_t)) {
 				fprintf(stderr, (waitcount++ % 10 > 0) ? "  ."
 					: "\nWarning: slow font initialization");
-#ifdef Q_OS_MAC
+#ifndef Q_OS_MAC
 				// OSX can be slow (>30 seconds?!) in determining font metrics
-				// Give it more time rather than failing after 1 second 
-				// Possibly this is only relevant to Qt5
-				GP_SLEEP(0.5);
-				continue;
+				// Always give it more time rather than failing after 1 second
+				// Everyone else can use --slow on the command line
+				if (slow_font_startup)
 #endif
+				{
+				    GP_SLEEP(0.5);
+				    continue;
+				}
 				return;
 			}
 			while (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t))
@@ -513,9 +564,33 @@ void qt_graphics()
 	// Initialize window
 	qt->out << GESetCurrentWindow << qt_optionWindowId;
 	qt->out << GEInitWindow;
+#ifdef _WIN32
+	// Let the terminal window know our PID
+	qt->out << GEPID << quint32(GetCurrentProcessId());
+#endif
 	qt->out << GEActivate;
 	qt->out << GETitle << qt_option->Title;
-	qt->out << GESetCtrl << qt_optionCtrl;
+
+	// We used to always send true or false; now it's a tristate
+	// only sent if there was an explicit set/unset in the "set term" command
+	if (qt_optionctrlq >= 0) {
+		qt->out << GESetCtrl << qt_optionCtrl;
+		qt_optionctrlq = -1;
+	}
+	if (qt_optionantialias >= 0) {
+		qt->out << GESetAntiAlias << qt_optionAntiAlias;
+		qt_optionantialias = -1;
+	}
+	if (qt_optionreplotonresize >= 0) {
+		qt->out << GESetReplotOnResize << qt_optionReplotOnResize;
+		qt_optionreplotonresize = -1;
+	}
+
+	if (qt_optionrounded >= 0) {
+	  qt->out << GESetRounded << qt_optionRounded;
+	  qt_optionrounded = -1;
+	}
+
 	qt->out << GESetWidgetSize << QSize(term->xmax, term->ymax)/qt_oversampling;
 	// Initialize the scene
 	qt->out << GESetSceneSize << QSize(term->xmax, term->ymax)/qt_oversampling;
@@ -524,6 +599,8 @@ void qt_graphics()
 	qt_sendFont();
 	term->v_tic = (unsigned int) (term->v_char/2.5);
 	term->h_tic = (unsigned int) (term->v_char/2.5);
+	// Tell user what scale factor relates term coordinates to pixels
+	term->tscale = qt_oversamplingF;
 
 	if (qt_setPosition)
 	{
@@ -536,7 +613,13 @@ void qt_graphics()
 void qt_text()
 {
 	if (qt_optionRaise)
+	{
+#ifdef _WIN32
+		// The qt graph window is in another process and we must explicitly allow it to "raise".
+		AllowSetForegroundWindow(qt->pid);
+#endif
 		qt->out << GERaise;
+	}
 	qt->out << GEDone;
 	qt_flushOutBuffer();
 }
@@ -546,7 +629,7 @@ void qt_text_wrapper()
 	if (!qt)
 		return;
 
-	// Remember scale to update the status bar while the plot is inactive
+	// Remember scale so that we can update the status bar while the plot is inactive
 	qt->out << GEScale;
 
 	const int axis_order[4] = {FIRST_X_AXIS, FIRST_Y_AXIS, SECOND_X_AXIS, SECOND_Y_AXIS};
@@ -566,13 +649,15 @@ void qt_text_wrapper()
 		qt->out << lower/qt_oversamplingF << scale/qt_oversamplingF;
 		qt->out << (axis_array[axis_order[i]].log ? axis_array[axis_order[i]].log_base : 0.);
 	}
+	// Flag whether this was a 3D plot (not mousable in 'persist' mode)
+	qt->out << qt_is_3Dplot;
+	qt_is_3Dplot = false;
 
 	qt_text();
 }
 
 void qt_reset()
 {
-	/// @todo
 }
 
 void qt_move(unsigned int x, unsigned int y)
@@ -590,7 +675,7 @@ void qt_enhanced_flush()
 	qt->out << GEEnhancedFlush << qt->enhancedFontName << qt->enhancedFontSize
 		<< (int)qt->enhancedFontStyle << (int)qt->enhancedFontWeight
 		<< qt->enhancedBase << qt->enhancedWidthFlag << qt->enhancedShowFlag
-		<< qt->enhancedOverprint 
+		<< qt->enhancedOverprint
 		<< qt->codec->toUnicode(qt->enhancedText);
 	qt->enhancedText.clear();
 }
@@ -611,6 +696,12 @@ void qt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN wid
 	qt->enhancedShowFlag  = showflag;
 	qt->enhancedOverprint = overprint;
 
+	// Baseline correction.  Surely Qt itself provides this somehow?
+	if (qt_max_pos_base < base)
+		qt_max_pos_base = base;
+	if (qt_max_neg_base > base)
+		qt_max_neg_base = base;
+
 	// strip Bold or Italic property out of font name
 	QString tempname = fontname;
 	if (tempname.contains(":italic", Qt::CaseInsensitive))
@@ -624,7 +715,7 @@ void qt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN wid
 	int sep = tempname.indexOf(":");
 	if (sep >= 0)
 		tempname.truncate(sep);
-	
+
 	// Blank font name means keep using the previous font
 	if (!tempname.isEmpty())
 		qt->enhancedFontName = tempname;
@@ -659,6 +750,9 @@ void qt_put_text(unsigned int x, unsigned int y, const char* string)
 	enhanced_fontscale = 1.0;
 	strncpy(enhanced_escape_format, "%c", sizeof(enhanced_escape_format));
 
+	// Baseline correction
+	qt_max_pos_base = qt_max_neg_base = 0.0;
+
 	// Set the recursion going. We say to keep going until a closing brace, but
 	// we don't really expect to find one.  If the return value is not the nul-
 	// terminator of the string, that can only mean that we did find an unmatched
@@ -674,26 +768,27 @@ void qt_put_text(unsigned int x, unsigned int y, const char* string)
 		// else carry on and process the rest of the string
 	}
 
+	// Baseline correction
+	y += qt_max_pos_base * 5;
+	y += qt_max_neg_base * 5;
+
 	qt->out << GEEnhancedFinish << qt_termCoord(x, y);
 }
 
 void qt_linetype(int lt)
 {
-	if (lt <= LT_NODRAW)
-		lt = LT_NODRAW; // background color
-
 	/* Version 5: dash pattern will be set later by term->dashtype */
 	if (lt == LT_AXIS)
 		qt->out << GEPenStyle << Qt::DotLine;
 	else if (lt == LT_NODRAW)
 		qt->out << GEPenStyle << Qt::NoPen;
-	else 
+	else
 		qt->out << GEPenStyle << Qt::SolidLine;
 
-	if ((lt-1) == LT_BACKGROUND) {
+	if (lt == LT_BACKGROUND) {
 		/* FIXME: Add parameter to this API to set the background color from the gnuplot end */
 		qt->out << GEBackgroundColor;
-	} else
+	} else if (lt > LT_NODRAW)
 		qt->out << GEPenColor << qt_colorList[lt % 9 + 3];
 }
 
@@ -740,7 +835,7 @@ void qt_dashtype(int type, t_dashtype *custom_dash_type)
 int qt_set_font(const char* font)
 {
 	ensureOptionsCreated();
-	int  qt_previousFontSize = qt->currentFontSize;
+	double qt_previousFontSize = qt->currentFontSize;
 	QString qt_previousFontName = qt->currentFontName;
 
 	if (font && (*font))
@@ -749,7 +844,7 @@ int qt_set_font(const char* font)
 		if (list.size() > 0)
 			qt->currentFontName = list[0];
 		if (list.size() > 1)
-			qt->currentFontSize = list[1].toInt();
+			qt->currentFontSize = list[1].toDouble();
 	} else {
 		qt->currentFontSize = qt_optionFontSize;
 		qt->currentFontName = qt_option->FontName;
@@ -763,7 +858,7 @@ int qt_set_font(const char* font)
 
 	/* Optimize by leaving early if there is no change */
 	if (qt->currentFontSize == qt_previousFontSize
-	&&  qt->currentFontName == qt->currentFontName) {
+	&&  qt->currentFontName == qt_previousFontName) {
 		return 1;
 	}
 
@@ -797,10 +892,10 @@ void qt_pointsize(double ptsize)
 
 void qt_linewidth(double lw)
 {
-	qt->out << GELineWidth << lw;
+	qt->out << GELineWidth << lw * qt_optionLineWidth;
 }
 
-int qt_text_angle(int angle)
+int qt_text_angle(float angle)
 {
 	qt->out << GETextAngle << double(angle);
 	return 1; // 1 means we can rotate
@@ -808,8 +903,14 @@ int qt_text_angle(int angle)
 
 void qt_fillbox(int style, unsigned int x, unsigned int y, unsigned int width, unsigned int height)
 {
+	// If y >= ymax then no box is drawn. So we clip.
+	unsigned int ytop = y + height;
+	if (ytop >= term->ymax) {
+	    ytop = term->ymax-1;
+	    height = ytop - y;
+	}
 	qt->out << GEBrushStyle << style;
-	qt->out << GEFillBox << QRect(qt_termCoord(x, y + height), QSize(width, height)/qt_oversampling);
+	qt->out << GEFillBox << QRect(qt_termCoord(x, ytop), QSize(width, height)/qt_oversampling);
 }
 
 int qt_make_palette(t_sm_palette* palette)
@@ -869,8 +970,8 @@ void qt_image(unsigned int M, unsigned int N, coordval* image, gpiPoint* corner,
 // box, with \r separating text above and below the point.
 void qt_put_tmptext(int n, const char str[])
 {
-    if (!qt)
-        return;
+	if (!qt)
+		return;
 
 	if (n == 0)
 		qt->out << GEStatusText << QString(str);
@@ -884,8 +985,8 @@ void qt_put_tmptext(int n, const char str[])
 
 void qt_set_cursor(int c, int x, int y)
 {
-    if (!qt)
-        return;
+	if (!qt)
+		return;
 
 	// Cancel zoombox when Echap is pressed
 	if (c == 0)
@@ -932,7 +1033,7 @@ void qt_set_clipboard(const char s[])
 int qt_waitforinput(int options)
 {
 #ifdef USE_MOUSE
-#ifndef WIN32
+#ifndef _WIN32
 	fd_set read_fds;
 	struct timeval one_msec;
 	int stdin_fd  = fileno(stdin);
@@ -966,8 +1067,16 @@ int qt_waitforinput(int options)
 		}
 
 		// Wait for input
-		if (select(socket_fd+1, &read_fds, NULL, NULL, timeout) < 0)
-		{
+		int n_changed_fds = select(socket_fd+1, &read_fds,
+						NULL,	// not watching for write-only
+						NULL,	// not watching for exceptions
+						timeout );
+
+		if (n_changed_fds < 0) {
+			if (errno == EINTR) {
+				wrap_readline_signal_handler();
+				continue;
+			}
 			// Display the error message except when Ctrl + C is pressed
 			if (errno != 4)
 				fprintf(stderr, "Qt terminal communication error: select() error %i %s\n", errno, strerror(errno));
@@ -1023,10 +1132,10 @@ int qt_waitforinput(int options)
 
 	if (options == TERM_ONLY_CHECK_MOUSING)
 		return '\0';
-	else
-		return getchar();
 
-#else // Windows console and wgnuplot
+	return getchar();
+
+#else // _WIN32, Windows console and wgnuplot
 #ifdef WGP_CONSOLE
 	int fd = fileno(stdin);
 #endif
@@ -1047,7 +1156,7 @@ int qt_waitforinput(int options)
 	if (options != TERM_ONLY_CHECK_MOUSING) { // NOTE: change this if used also for the caca terminal
 #ifdef WGP_CONSOLE
 		if (!isatty(fd))
-			h[0] = CreateThread(NULL, 0, stdin_pipe_reader, NULL, 0, NULL);
+			h[0] = init_pipe_input();
 		else
 #endif
 			h[0] = GetStdHandle(STD_INPUT_HANDLE);
@@ -1073,7 +1182,7 @@ int qt_waitforinput(int options)
 
 	do {
 		int waitResult = -1;
-		
+
 #ifndef WGP_CONSOLE
 		// Process pending key events of the text console
 		if (kbhit() && (options != TERM_ONLY_CHECK_MOUSING))
@@ -1084,21 +1193,17 @@ int qt_waitforinput(int options)
 		if ((idx_socket != -1) && // (qt != NULL)) &&
 			(qt->socket.waitForReadyRead(0)) && (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t)))
 			waitResult = idx_socket; // data already available
-		
-		// Wait for a new event 
+
+		// Wait for a new event
 		if ((waitResult == -1) && (options != TERM_ONLY_CHECK_MOUSING))
 			waitResult = MsgWaitForMultipleObjects(idx, h, FALSE, INFINITE, QS_ALLINPUT);  // wait for new data
-		
+
 		if ((waitResult == idx_stdin) && (idx_stdin != -1)) { // console windows or caca terminal (TBD)
 #ifdef WGP_CONSOLE
 			if (!isatty(fd)) {
-				DWORD dw;
-
-				GetExitCodeThread(h[0], &dw);
-				CloseHandle(h[0]);
-				c = dw;
+				c = next_pipe_input();
 				quitLoop = true;
-			} else 
+			} else
 #endif
 			{
 				c = ConsoleReadCh();
@@ -1113,6 +1218,7 @@ int qt_waitforinput(int options)
 			// are received, only transmit the last one.
 			gp_event_t tempEvent;
 			tempEvent.type = -1;
+			int size = qt->socket.bytesAvailable();
 			while (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t)) {
 				struct gp_event_t event;
 				qt->socket.read((char*) &event, sizeof(gp_event_t));
@@ -1131,6 +1237,10 @@ int qt_waitforinput(int options)
 					}
 				}
 			}
+			// If the native pipe handle signalled new data, but the QtLocalSocket
+			// object has no data available, release the CPU for a little while.
+			if (size == 0)
+				Sleep(100);
 			// Replay move event
 			if (tempEvent.type == GE_motion)
 				qt_processTermEvent(&tempEvent);
@@ -1161,17 +1271,16 @@ int qt_waitforinput(int options)
 		}
 	} while (!quitLoop);
 
-
 #ifndef WGP_CONSOLE
 	if (options != TERM_ONLY_CHECK_MOUSING)
 		TextStopEditing(&textwin);
-	
+
 	// This happens if neither the qt queue is alive, nor there is a console window.
 	if ((options != TERM_ONLY_CHECK_MOUSING) && !waitOK)
 		return getchar();
 #endif
 	return c;
-#endif // WIN32
+#endif // _WIN32
 #else
 	return getchar();
 #endif // USE_MOUSE
@@ -1194,9 +1303,9 @@ void qt_atexit()
 	else
 		qt->out << GEExit;
 	qt_flushOutBuffer();
-        
-        delete qt;
-        qt = NULL;
+
+	delete qt;
+	qt = NULL;
 
 	delete qt_option;
 	qt_option = NULL;
@@ -1224,7 +1333,14 @@ enum QT_id {
 	QT_DASH,
 	QT_DASHLENGTH,
 	QT_SOLID,
-	QT_OTHER
+	QT_LINEWIDTH,
+	QT_OTHER,
+	QT_ROUNDED,
+	QT_NOROUNDED,
+	QT_ANTIALIAS,
+	QT_NOANTILIAS,
+	QT_REPLOTONRESIZE,
+	QT_NOREPLOTONRESIZE
 };
 
 static struct gen_table qt_opts[] = {
@@ -1240,12 +1356,20 @@ static struct gen_table qt_opts[] = {
 	{"norai$se",    QT_NORAISE},
 	{"ct$rlq",      QT_CTRL},
 	{"noct$rlq",    QT_NOCTRL},
+	{"rou$nded",    QT_ROUNDED},
+	{"butt",        QT_NOROUNDED},
+	{"an$tialias",  QT_ANTIALIAS},
+	{"noan$tialias",QT_NOANTILIAS},
+	{"rep$lotonresize", QT_REPLOTONRESIZE},
+	{"norep$lotonresize", QT_NOREPLOTONRESIZE},
 	{"ti$tle",      QT_TITLE},
 	{"cl$ose",      QT_CLOSE},
-	{"dash$ed",	QT_DASH},
-	{"dashl$ength",	QT_DASHLENGTH},
-	{"dl",		QT_DASHLENGTH},
-	{"solid",	QT_SOLID},
+	{"dash$ed",     QT_DASH},
+	{"dashl$ength", QT_DASHLENGTH},
+	{"dl",          QT_DASHLENGTH},
+	{"solid",       QT_SOLID},
+	{"line$width",  QT_LINEWIDTH},
+	{"lw",          QT_LINEWIDTH},
 	{NULL,          QT_OTHER}
 };
 
@@ -1268,8 +1392,13 @@ void qt_options()
 	bool set_widget = false;
 	bool set_dash = false;
 	bool set_dashlength = false;
+	bool set_linewidth = false;
+	bool set_rounded = false;
+	bool set_antialias = false;
+	bool set_replotonresize = false;
+	int previous_WindowId = qt_optionWindowId;
 
-#ifndef WIN32
+#ifndef _WIN32
 	if (term_interlock != NULL && term_interlock != (void *)qt_init) {
 		term = NULL;
 		int_error(NO_CARET, "The qt terminal cannot be used in a wxt session");
@@ -1300,8 +1429,8 @@ void qt_options()
 				QStringList list = fontSettings.split(',');
 				if ((list.size() > 0) && !list[0].isEmpty())
 					qt_option->FontName = list[0];
-				if ((list.size() > 1) && (list[1].toInt() > 0))
-					qt_optionFontSize = list[1].toInt();
+				if ((list.size() > 1) && (list[1].toDouble() > 0))
+					qt_optionFontSize = list[1].toDouble();
 			}
 			free(s);
 			break;
@@ -1359,6 +1488,31 @@ void qt_options()
 			SETCHECKDUP(set_ctrl);
 			qt_optionCtrl = false;
 			break;
+
+		case QT_ROUNDED:
+			SETCHECKDUP(set_rounded);
+			qt_optionRounded = true;
+			break;
+		case QT_NOROUNDED:
+			SETCHECKDUP(set_rounded);
+			qt_optionRounded = false;
+			break;
+		case QT_ANTIALIAS:
+			SETCHECKDUP(set_antialias);
+			qt_optionAntiAlias = true;
+			break;
+		case QT_NOANTILIAS:
+			SETCHECKDUP(set_antialias);
+			qt_optionAntiAlias = false;
+			break;
+		case QT_REPLOTONRESIZE:
+			SETCHECKDUP(set_replotonresize);
+			qt_optionReplotOnResize = true;
+			break;
+		case QT_NOREPLOTONRESIZE:
+			SETCHECKDUP(set_replotonresize);
+			qt_optionReplotOnResize = false;
+			break;
 		case QT_TITLE:
 			SETCHECKDUP(set_title);
 			if (!(s = try_to_get_string()))
@@ -1384,6 +1538,10 @@ void qt_options()
 			// qt_optionDash = false;
 			c_token++;
 			break;
+		case QT_LINEWIDTH:
+			SETCHECKDUP(set_linewidth);
+			qt_optionLineWidth = real_expression();
+			break;
 		case QT_OTHER:
 		default:
 			qt_optionWindowId = int_expression();
@@ -1397,6 +1555,14 @@ void qt_options()
 			int_error(c_token-1, "Duplicated or contradicting arguments in qt term options.");
 	}
 
+	// We want this to happen immediately, hence the flush command.
+	// We don't want it to change the _current_ window, just close an old one.
+	if (set_close && qt) {
+		qt->out << GECloseWindow << qt_optionWindowId;
+		qt_flushOutBuffer();
+		qt_optionWindowId = previous_WindowId;
+	}
+
 	// Save options back into options string in normalized format
 	QString termOptions = QString::number(qt_optionWindowId);
 
@@ -1424,15 +1590,35 @@ void qt_options()
 		qt_setPosition = true;
 	}
 
+	termOptions += " font \"" + fontSettings + '"';
+
 	if (set_enhanced) termOptions += qt_optionEnhanced ? " enhanced" : " noenhanced";
-	                  termOptions += " font \"" + fontSettings + '"';
+	if (set_linewidth) termOptions += " linewidth " + QString::number(qt_optionLineWidth);
 	if (set_dashlength) termOptions += " dashlength " + QString::number(qt_optionDashLength);
 	if (set_widget)   termOptions += " widget \"" + qt_option->Widget + '"';
 	if (set_persist)  termOptions += qt_optionPersist ? " persist" : " nopersist";
 	if (set_raise)    termOptions += qt_optionRaise ? " raise" : " noraise";
 	if (set_ctrl)     termOptions += qt_optionCtrl ? " ctrl" : " noctrl";
 
-	if (set_close && qt) qt->out << GECloseWindow << qt_optionWindowId;
+	/// Only send the ctrlQ option if it is explicitly given in "set term qt {no}ctrlq"
+	if (set_ctrl)
+	    qt_optionctrlq = qt_optionCtrl;
+
+	if (set_rounded) termOptions += qt_optionRounded ? " rounded" : " butt";
+
+	if (set_rounded)
+	  qt_optionrounded = qt_optionRounded;
+
+	if (set_antialias) termOptions += qt_optionAntiAlias ? " antialias" : " noantialias";
+
+	if (set_antialias)
+	  qt_optionantialias = qt_optionAntiAlias;
+
+	if (set_replotonresize) termOptions += qt_optionReplotOnResize ? " replotonresize" : " noreplotonresize";
+
+	if (set_replotonresize)
+	  qt_optionreplotonresize = qt_optionReplotOnResize;
+
 
 	/// @bug change Utf8 to local encoding
 	strncpy(term_options, termOptions.toUtf8().data(), MAX_LINE_LEN);
@@ -1440,12 +1626,12 @@ void qt_options()
 
 void qt_layer( t_termlayer syncpoint )
 {
-    static int current_plotno = 0;
+	static int current_plotno = 0;
 	if (!qt)
 		return;
 
-    /* We must ignore all syncpoints that we don't recognize */
-    switch (syncpoint) {
+	/* We must ignore all syncpoints that we don't recognize */
+	switch (syncpoint) {
 	case TERM_LAYER_BEFORE_PLOT:
 		current_plotno++;
 		qt->out << GEPlotNumber << current_plotno; break;
@@ -1467,9 +1653,11 @@ void qt_layer( t_termlayer syncpoint )
 		qt->out << GELayer << QTLAYER_END_KEYSAMPLE; break;
 	case TERM_LAYER_BEFORE_ZOOM:
 		qt->out << GELayer << QTLAYER_BEFORE_ZOOM; break;
-    	default:
+	case TERM_LAYER_3DPLOT:
+		qt_is_3Dplot = true; break;
+	default:
 		break;
-    }
+	}
 }
 
 void qt_hypertext( int type, const char *text )
@@ -1478,12 +1666,16 @@ void qt_hypertext( int type, const char *text )
 		qt->out << GEHypertext << qt->codec->toUnicode(text);
 }
 
-#ifdef EAM_BOXED_TEXT
 void qt_boxed_text(unsigned int x, unsigned int y, int option)
 {
-	qt->out << GETextBox << qt_termCoordF(x, y) << option;
+	if (option == TEXTBOX_MARGINS) {
+	    // slightly largin y margin to allow for descenders
+	    double xmargin = (double)x/(100*qt_oversamplingF);
+	    double ymargin = (double)y/(100*qt_oversamplingF) + 0.75/qt_oversamplingF;
+	    qt->out << GETextBox << QPointF(xmargin, ymargin) << option;
+	} else
+	    qt->out << GETextBox << qt_termCoordF(x, y) << option;
 }
-#endif
 
 void qt_modify_plots(unsigned int ops, int plotno)
 {
