@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: stats.c,v 1.14.2.6 2015/08/01 05:07:25 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - stats.c */
 
 /*
@@ -40,38 +36,42 @@ static char *RCSid() { return RCSid("$Id: stats.c,v 1.14.2.6 2015/08/01 05:07:25
 #include "command.h"
 #include "datafile.h"
 #include "gadgets.h"  /* For polar and parametric flags */
+#include "gplocale.h" /* For locale handling */
 #include "matrix.h"   /* For vector allocation */
 #include "scanner.h"  /* To check for legal prefixes */
-#include "variable.h" /* For locale handling */
+#include "voxelgrid.h"
 
 #include "stats.h"
 
 #define INITIAL_DATA_SIZE (4096)   /* initial size of data arrays */
 
-static int comparator __PROTO(( const void *a, const void *b ));
-static struct file_stats analyze_file __PROTO(( long n, int outofrange, int invalid, int blank, int dblblank ));
-static struct sgl_column_stats analyze_sgl_column __PROTO(( double *data, long n, long nr ));
-static struct two_column_stats analyze_two_columns __PROTO(( double *x, double *y,
-							     struct sgl_column_stats res_x,
-							     struct sgl_column_stats res_y,
-							     long n ));
+static int comparator( const void *a, const void *b );
+static struct file_stats analyze_file( long n, int outofrange, int invalid, int blank, int dblblank, int headers );
+static struct sgl_column_stats analyze_sgl_column( double *data, long n, long nr );
+static struct two_column_stats analyze_two_columns( double *x, double *y,
+						     struct sgl_column_stats res_x,
+						     struct sgl_column_stats res_y,
+						     long n );
 
-static void ensure_output __PROTO((void));
-static char* fmt __PROTO(( char *buf, double val ));
-static void sgl_column_output_nonformat __PROTO(( struct sgl_column_stats s, char *x ));
-static void file_output __PROTO(( struct file_stats s ));
-static void sgl_column_output __PROTO(( struct sgl_column_stats s, long n ));
-static void two_column_output __PROTO(( struct sgl_column_stats x,
-					struct sgl_column_stats y,
-					struct two_column_stats xy, long n));
+static void ensure_output(void);
+static char* fmt( char *buf, double val );
+static void sgl_column_output_nonformat( struct sgl_column_stats s, char *x );
+static void file_output( struct file_stats s );
+static void sgl_column_output( struct sgl_column_stats s, long n );
+static void two_column_output( struct sgl_column_stats x,
+				struct sgl_column_stats y,
+				struct two_column_stats xy, long n);
 
-static void create_and_set_var __PROTO(( double val, char *prefix,
-					 char *base, char *suffix ));
+static void clear_one_var( char *prefix, char *base );
+static void clear_stats_variables( char *prefix );
 
-static void sgl_column_variables __PROTO(( struct sgl_column_stats res,
-					   char *prefix, char *postfix ));
+static void create_and_set_var( double val, char *prefix, char *base, char *suffix );
+static void create_and_set_int_var( int ival, char *prefix, char *base, char *suffix );
+static void create_and_store_var( t_value *data, char *prefix, char *base, char *suffix );
 
-static TBOOLEAN validate_data __PROTO((double v, AXIS_INDEX ax));
+static void sgl_column_variables( struct sgl_column_stats res, char *prefix, char *postfix );
+
+static TBOOLEAN validate_data(double v, AXIS_INDEX ax);
 
 /* =================================================================
    Data Structures
@@ -90,6 +90,8 @@ struct file_stats {
     long invalid;
     long outofrange;
     long blocks;  /* blocks are separated by double blank lines */
+    long header_records;
+    int  columns;
 };
 
 struct sgl_column_stats {
@@ -160,7 +162,7 @@ comparator( const void *a, const void *b )
 }
 
 static struct file_stats
-analyze_file( long n, int outofrange, int invalid, int blank, int dblblank )
+analyze_file( long n, int outofrange, int invalid, int blank, int dblblank, int headers )
 {
     struct file_stats res;
 
@@ -169,6 +171,8 @@ analyze_file( long n, int outofrange, int invalid, int blank, int dblblank )
     res.blanks  = blank;
     res.blocks  = dblblank + 1;  /* blocks are separated by dbl blank lines */
     res.outofrange = outofrange;
+    res.header_records = headers;
+    res.columns = df_last_col;
 
     return res;
 }
@@ -203,7 +207,7 @@ analyze_sgl_column( double *data, long n, long nc )
     }
 
     /* Mean and centre of gravity */
-    for( i=0; i<n; i++ ) {
+    for (i=0; i<n; i++) {
 	s  += data[i];
 	s2 += data[i]*data[i];
 	if ( nc > 0 ) {
@@ -217,7 +221,7 @@ analyze_sgl_column( double *data, long n, long nc )
     res.sum_sq = s2;
 
     /* Standard deviation, mean absolute deviation, skewness, and kurtosis */
-    for( i=0; i<n; i++ ) {
+    for (i=0; i<n; i++) {
 	double t = data[i] - res.mean;
 	ad += fabs(t);
 	d  += t;
@@ -245,7 +249,7 @@ analyze_sgl_column( double *data, long n, long nc )
     /* sample standard deviation */
     res.ssd = res.stddev * sqrt((double)(n) / (double)(n-1));
 
-    for( i=0; i<n; i++ ) {
+    for (i=0; i<n; i++) {
 	tmp[i].val = data[i];
 	tmp[i].index = i;
     }
@@ -296,7 +300,7 @@ analyze_two_columns( double *x, double *y,
     double s = 0;
     double ssyy, ssxx, ssxy;
 
-    for( i=0; i<n; i++ ) {
+    for (i=0; i<n; i++) {
 	s += x[i] * y[i];
     }
     res.sum_xy = s;
@@ -386,6 +390,7 @@ file_output( struct file_stats s )
 	fprintf( print_out, "%s\t%ld\n", "invalid", s.invalid );
 	fprintf( print_out, "%s\t%ld\n", "blanks", s.blanks );
 	fprintf( print_out, "%s\t%ld\n", "blocks", s.blocks );
+	fprintf( print_out, "%s\t%ld\n", "headers", s.header_records );
 	fprintf( print_out, "%s\t%ld\n", "outofrange", s.outofrange );
 	return;
     }
@@ -396,6 +401,7 @@ file_output( struct file_stats s )
     fprintf( print_out, "  Records:           %*ld\n", width, s.records );
     fprintf( print_out, "  Out of range:      %*ld\n", width, s.outofrange );
     fprintf( print_out, "  Invalid:           %*ld\n", width, s.invalid );
+    fprintf( print_out, "  Header records:    %*ld\n", width, s.header_records );
     fprintf( print_out, "  Blank:             %*ld\n", width, s.blanks );
     fprintf( print_out, "  Data Blocks:       %*ld\n", width, s.blocks );
 }
@@ -586,16 +592,100 @@ two_column_output( struct sgl_column_stats x,
    Variable Handling
    ================================================================= */
 
+static void
+clear_one_var( char *prefix, char *base )
+{
+    int len;
+    char *varname;
+
+    len = strlen(prefix) + strlen(base) + 2;
+    varname = (char *)gp_alloc( len, "create_and_set_var" );
+    sprintf( varname, "%s_%s", prefix, base );
+
+    del_udv_by_name(varname, TRUE);
+
+    free( varname );
+}
+
+static void
+clear_stats_variables( char *prefix )
+{
+    /* file variables */
+    clear_one_var( prefix, "records" );
+    clear_one_var( prefix, "invalid" );
+    clear_one_var( prefix, "headers" );
+    clear_one_var( prefix, "blank" );
+    clear_one_var( prefix, "blocks" );
+    clear_one_var( prefix, "outofrange" );
+    clear_one_var( prefix, "columns" );
+
+    /* one column variables */
+    clear_one_var( prefix, "mean" );
+    clear_one_var( prefix, "stddev" );
+    clear_one_var( prefix, "ssd" );
+    clear_one_var( prefix, "skewness" );
+    clear_one_var( prefix, "kurtosis" );
+    clear_one_var( prefix, "adev" );
+    clear_one_var( prefix, "mean_err" );
+    clear_one_var( prefix, "stddev_err" );
+    clear_one_var( prefix, "skewness_err" );
+    clear_one_var( prefix, "kurtosis_err" );
+    clear_one_var( prefix, "sum" );
+    clear_one_var( prefix, "sumsq" );
+    clear_one_var( prefix, "min" );
+    clear_one_var( prefix, "max" );
+    clear_one_var( prefix, "median" );
+    clear_one_var( prefix, "lo_quartile" );
+    clear_one_var( prefix, "up_quartile" );
+    clear_one_var( prefix, "index_min" );
+    clear_one_var( prefix, "index_max" );
+
+    /* matrix variables */
+    clear_one_var( prefix, "index_min_x" );
+    clear_one_var( prefix, "index_min_y" );
+    clear_one_var( prefix, "index_max_x" );
+    clear_one_var( prefix, "index_max_y" );
+    clear_one_var( prefix, "size_x" );
+    clear_one_var( prefix, "size_y" );
+
+    /* two column variables */
+    clear_one_var( prefix, "slope" );
+    clear_one_var( prefix, "intercept" );
+    clear_one_var( prefix, "slope_err" );
+    clear_one_var( prefix, "intercept_err" );
+    clear_one_var( prefix, "correlation" );
+    clear_one_var( prefix, "sumxy" );
+    clear_one_var( prefix, "pos_min_y" );
+    clear_one_var( prefix, "pos_max_y" );
+
+    /* column headers */
+    clear_one_var( prefix, "column_header" );
+
+}
+
 static void
 create_and_set_var( double val, char *prefix, char *base, char *suffix )
+{
+    t_value data;
+    Gcomplex( &data, val, 0.0 ); /* data is complex, real=val, imag=0.0 */
+    create_and_store_var( &data, prefix, base, suffix );
+}
+
+static void
+create_and_set_int_var( int ival, char *prefix, char *base, char *suffix )
+{
+    t_value data;
+    Ginteger( &data, ival );
+    create_and_store_var( &data, prefix, base, suffix );
+}
+
+static void
+create_and_store_var( t_value *data, char *prefix, char *base, char *suffix )
 {
     int len;
     char *varname;
     struct udvt_entry *udv_ptr;
 
-    t_value data;
-    Gcomplex( &data, val, 0.0 ); /* data is complex, real=val, imag=0.0 */
-
     /* In case prefix (or suffix) is NULL - make them empty strings */
     prefix = prefix ? prefix : "";
     suffix = suffix ? suffix : "";
@@ -609,8 +699,7 @@ create_and_set_var( double val, char *prefix, char *base, char *suffix )
      * its own copy of the varname.
      */
     udv_ptr = add_udv_by_name(varname);
-    udv_ptr->udv_value = data;
-    udv_ptr->udv_undef = FALSE;
+    udv_ptr->udv_value = *data;
 
     free( varname );
 }
@@ -619,12 +708,27 @@ static void
 file_variables( struct file_stats s, char *prefix )
 {
     /* Suffix does not make sense here! */
-    create_and_set_var( s.records, prefix, "records", "" );
-    create_and_set_var( s.invalid, prefix, "invalid", "" );
-    create_and_set_var( s.blanks,  prefix, "blank",   "" );
-    create_and_set_var( s.blocks,  prefix, "blocks",  "" );
-    create_and_set_var( s.outofrange, prefix, "outofrange", "" );
-    create_and_set_var( df_last_col, prefix, "columns", "" );
+    create_and_set_int_var( s.records, prefix, "records", "" );
+    create_and_set_int_var( s.invalid, prefix, "invalid", "" );
+    create_and_set_int_var( s.header_records, prefix, "headers", "" );
+    create_and_set_int_var( s.blanks,  prefix, "blank",   "" );
+    create_and_set_int_var( s.blocks,  prefix, "blocks",  "" );
+    create_and_set_int_var( s.outofrange, prefix, "outofrange", "" );
+    create_and_set_int_var( s.columns, prefix, "columns", "" );
+
+    /* copy column headers to an array */
+    if (df_columnheaders) {
+	int i;
+	t_value headers;
+	t_value *A = gp_alloc((s.columns+1) * sizeof(t_value), "column_headers");
+
+	Ginteger(&A[0], s.columns);
+	for (i = 1; i <= s.columns; i++)
+	    Gstring(&A[i], gp_strdup(df_retrieve_columnhead(i)));
+	headers.type = ARRAY;
+	headers.v.value_array = A;
+	create_and_store_var( &headers, prefix, "column_header", "" );
+    }
 }
 
 static void
@@ -650,18 +754,18 @@ sgl_column_variables( struct sgl_column_stats s, char *prefix, char *suffix )
 
     /* If data set is matrix */
     if ( s.sx > 0 ) {
-	create_and_set_var( (s.min.index) % s.sx, prefix, "index_min_x", suffix );
-	create_and_set_var( (s.min.index) / s.sx, prefix, "index_min_y", suffix );
-	create_and_set_var( (s.max.index) % s.sx, prefix, "index_max_x", suffix );
-	create_and_set_var( (s.max.index) / s.sx, prefix, "index_max_y", suffix );
-	create_and_set_var( s.sx, prefix, "size_x", suffix );
-	create_and_set_var( s.sy, prefix, "size_y", suffix );
+	create_and_set_int_var( (s.min.index) % s.sx, prefix, "index_min_x", suffix );
+	create_and_set_int_var( (s.min.index) / s.sx, prefix, "index_min_y", suffix );
+	create_and_set_int_var( (s.max.index) % s.sx, prefix, "index_max_x", suffix );
+	create_and_set_int_var( (s.max.index) / s.sx, prefix, "index_max_y", suffix );
+	create_and_set_int_var( s.sx, prefix, "size_x", suffix );
+	create_and_set_int_var( s.sy, prefix, "size_y", suffix );
     } else {
 	create_and_set_var( s.median,         prefix, "median",      suffix );
 	create_and_set_var( s.lower_quartile, prefix, "lo_quartile", suffix );
 	create_and_set_var( s.upper_quartile, prefix, "up_quartile", suffix );
-	create_and_set_var( s.min.index, prefix, "index_min", suffix );
-	create_and_set_var( s.max.index, prefix, "index_max", suffix );
+	create_and_set_int_var( s.min.index, prefix, "index_min", suffix );
+	create_and_set_int_var( s.max.index, prefix, "index_max", suffix );
     }
 }
 
@@ -729,6 +833,7 @@ statsrequest(void)
     long invalid;          /* number of missing/invalid records */
     long blanks;           /* number of blank lines */
     long doubleblanks;     /* number of repeated blank lines */
+    long header_records;   /* number of records treated as headers rather than data */
     long out_of_range;     /* number pts rejected, because out of range */
 
     struct file_stats res_file;
@@ -736,22 +841,25 @@ statsrequest(void)
     struct two_column_stats res_xy = {0};
 
     /* Vars for variable handling */
-    static char *prefix = NULL;       /* prefix for user-defined vars names */
+    static char *prefix = NULL;	/* prefix for user-defined vars names */
+    TBOOLEAN prefix_from_columnhead = FALSE;
 
     /* Vars that control output */
-    TBOOLEAN do_output = TRUE;     /* Generate formatted output */
+    TBOOLEAN do_output = TRUE;	/* Generate formatted output */
+    TBOOLEAN array_data = FALSE;
 
     c_token++;
 
     /* Parse ranges */
-    AXIS_INIT2D(FIRST_X_AXIS, 0);
-    AXIS_INIT2D(FIRST_Y_AXIS, 0);
+    axis_init(&axis_array[FIRST_X_AXIS], FALSE);
+    axis_init(&axis_array[FIRST_Y_AXIS], FALSE);
     parse_range(FIRST_X_AXIS);
     parse_range(FIRST_Y_AXIS);
 
     /* Initialize */
     invalid = 0;          /* number of missing/invalid records */
     blanks = 0;           /* number of blank lines */
+    header_records = 0;   /* number of records treated as headers rather than data */
     doubleblanks = 0;     /* number of repeated blank lines */
     out_of_range = 0;     /* number pts rejected, because out of range */
     n = 0;                /* number of records retained */
@@ -759,13 +867,15 @@ statsrequest(void)
 
     free(data_x);
     free(data_y);
-    data_x = vec(max_n);       /* start with max. value */
+    data_x = vec(max_n);  /* start with max. value */
     data_y = vec(max_n);
+    free(prefix);
+    prefix = NULL;
 
     if ( !data_x || !data_y )
       int_error( NO_CARET, "Internal error: out of memory in stats" );
 
-    n = invalid = blanks = doubleblanks = out_of_range = 0;
+    n = invalid = blanks = header_records = doubleblanks = out_of_range = 0;
 
     /* Get filename */
     i = c_token;
@@ -783,15 +893,27 @@ statsrequest(void)
      * to set the effective number of columns to 1.
      */
     if (TRUE) {
-	columns = df_open(file_name, 2, NULL); /* up to 2 using specs allowed */
+	df_set_plot_mode(MODE_PLOT);		/* Used for matrix datafiles */
+	columns = df_open(file_name, 2, NULL);	/* up to 2 using specs allowed */
 
-	if (columns < 0)
-	    int_error(NO_CARET, "Can't read data file");
+	/* "stats <badfilename> nooutput"
+	 * allows user to test for existance of a file without generating a fatal error.
+	 * The 'nooutput' keyword suppresses the resulting error message as well.
+	 */
+	if (columns < 0) {
+	    fill_gpval_integer("GPVAL_ERRNO", 1);
+	    while (!END_OF_COMMAND)
+		if (almost_equals(c_token++, "noout$put"))
+		    do_output = FALSE;
+	    if (do_output)
+		fprintf(stderr, "Cannot find or open file \"%s\"\n", file_name);
+	    goto stats_cleanup;
+	}
+
+	if (df_array && columns == 0)
+	    array_data = TRUE;
 
 	/* For all these below: we could save the state, switch off, then restore */
-	if ( axis_array[FIRST_X_AXIS].log || axis_array[FIRST_Y_AXIS].log )
-	    int_error( NO_CARET, "Stats command not available with logscale active");
-
 	if (axis_array[FIRST_X_AXIS].datatype == DT_TIMEDATE
 	||  axis_array[FIRST_Y_AXIS].datatype == DT_TIMEDATE )
 	    int_error( NO_CARET, "Stats command not available in timedata mode");
@@ -802,6 +924,57 @@ statsrequest(void)
 	if ( parametric )
 	    int_error( NO_CARET, "Stats command not available in parametric mode" );
 
+	/* Parse the remainder of the command line */
+	while( !(END_OF_COMMAND) ) {
+	    if ( almost_equals( c_token, "out$put" ) ) {
+		    do_output = TRUE;
+		    c_token++;
+
+	    } else if ( almost_equals( c_token, "noout$put" ) ) {
+		    do_output = FALSE;
+		    c_token++;
+
+	    } else if ( almost_equals(c_token, "pre$fix")
+		   ||   equals(c_token, "name")) {
+		c_token++;
+		if (almost_equals(c_token,"col$umnheader")) {
+		    df_set_key_title_columnhead(NULL);
+		    prefix_from_columnhead = TRUE;
+		    continue;
+		}
+		prefix = try_to_get_string();
+		if (!legal_identifier(prefix) || !strcmp ("GPVAL_", prefix))
+		    int_error( --c_token, "illegal prefix" );
+
+	    }  else {
+		int_error( c_token, "Unrecognized stats option");
+	    }
+	}
+
+	/* Clear any previous variables STATS_* so that if we exit early */
+	/* they cannot be mistaken as resulting from the current analysis. */
+	clear_stats_variables(prefix ? prefix : "STATS");
+
+	/* Special case for voxel grid stats: "stats $vgrid {name <prefix>} */
+	if (df_voxelgrid) {
+	    vgrid *vgrid = get_vgrid_by_name(file_name)->udv_value.v.vgrid;
+	    int N, nonzero;
+
+	    vgrid_stats(vgrid);
+	    N = vgrid->size;
+	    nonzero = N*N*N - vgrid->nzero;
+
+	    if (!prefix)
+		prefix = gp_strdup("STATS");
+	    create_and_set_var( vgrid->mean_value, prefix, "_mean",   "" );
+	    create_and_set_var( vgrid->stddev, prefix, "_stddev", "" );
+	    create_and_set_var( vgrid->sum, prefix, "_sum", "" );
+	    create_and_set_var( vgrid->min_value, prefix, "_min", "" );
+	    create_and_set_var( vgrid->max_value, prefix, "_max", "" );
+	    create_and_set_var( nonzero, prefix, "_nonzero", "" );
+	    goto stats_cleanup;
+	}
+
 	/* If the user has set an explicit locale for numeric input, apply it */
 	/* here so that it affects data fields read from the input file. */
 	set_numeric_locale();
@@ -828,6 +1001,12 @@ statsrequest(void)
 		}
 	    } /* if (need to extend storage space) */
 
+	    /* ascii "matrix" input from df_readline via df_readbinary does not
+	     * flag NaN values so we must check each returned value here
+	     */
+	    if (df_matrix && (i == 2) && isnan(v[1]))
+		i = DF_UNDEFINED;
+
 	    switch (i) {
 	    case DF_MISSING:
 	    case DF_UNDEFINED:
@@ -843,8 +1022,12 @@ statsrequest(void)
 	      doubleblanks += 1;
 	      continue;
 
+	    case DF_COLUMN_HEADERS:
+	      header_records += 1;
+	      continue;
+
 	    case 0:
-	      int_error( NO_CARET, "bad data on line %d of file %s",
+	      int_warn( NO_CARET, "bad data on line %d of file %s",
 	  		df_line_number, df_filename ? df_filename : "" );
 	      break;
 
@@ -869,6 +1052,11 @@ statsrequest(void)
 	      }
 	      columns = 2;
 	      break;
+
+	    default: /* Who are these? */
+	      FPRINTF((stderr,"unhandled return code %d from df_readline\n", i));
+	      break;
+
 	    }
 	} /* end-while : done reading file */
 	df_close();
@@ -883,37 +1071,24 @@ statsrequest(void)
 
     /* No data! Try to explain why. */
     if ( n == 0 ) {
-	if ( out_of_range > 0 )
-	    int_error( NO_CARET, "All points out of range" );
+	if (out_of_range > 0)
+	    int_warn( NO_CARET, "All points out of range" );
 	else
-	    int_error( NO_CARET, "No valid data points found in file" );
+	    int_warn( NO_CARET, "No valid data points found in file" );
+	goto stats_cleanup;
     }
 
-    /* Parse the remainder of the command line: 0 to 2 tokens possible */
-    while( !(END_OF_COMMAND) ) {
-	if ( almost_equals( c_token, "out$put" ) ) {
-		do_output = TRUE;
-		c_token++;
-
-	} else if ( almost_equals( c_token, "noout$put" ) ) {
-		do_output = FALSE;
-		c_token++;
-
-	} else if ( almost_equals(c_token, "pre$fix")
-	       ||   equals(c_token, "name")) {
-	    c_token++;
-	    free ( prefix );
-	    prefix = try_to_get_string();
-	    if (!legal_identifier(prefix) || !strcmp ("GPVAL_", prefix))
-		int_error( --c_token, "illegal prefix" );
-
-	}  else {
-	    int_error( c_token, "Expecting [no]output or prefix");
+    /* The analysis variables are named STATS_* unless the user either */
+    /* gave a specific name or told us to use a columnheader.          */
+    if (!prefix && prefix_from_columnhead && df_key_title && *df_key_title) {
+	prefix = gp_strdup(df_key_title);
+	squash_spaces(prefix, 0);
+	if (!legal_identifier(prefix)) {
+	    int_warn(NO_CARET, "columnhead %s is not a valid prefix", prefix ? prefix : "");
+	    free(prefix);
+	    prefix = NULL;
 	}
-
     }
-
-    /* Set defaults if not explicitly set by user */
     if (!prefix)
 	prefix = gp_strdup("STATS_");
     i = strlen(prefix);
@@ -923,12 +1098,16 @@ statsrequest(void)
     }
 
     /* Do the actual analysis */
-    res_file = analyze_file( n, out_of_range, invalid, blanks, doubleblanks );
+    res_file = analyze_file( n, out_of_range, invalid, blanks, doubleblanks, header_records );
 
     /* Jan 2015: Revised detection and handling of matrix data */
+    if (array_data)
+	res_file.columns = columns = 1;
+
     if (df_matrix) {
-	int nc = df_bin_record[df_num_bin_records-1].scan_dim[0];
+	int nc = df_bin_record[df_num_bin_records-1].submatrix_ncols;
 	res_y = analyze_sgl_column( data_y, n, nc );
+	res_file.columns = nc;
 	columns = 1;
 
     } else if (columns == 1) {
@@ -967,6 +1146,7 @@ statsrequest(void)
     }
 
     /* Cleanup */
+    stats_cleanup:
 
     free(data_x);
     free(data_y);
