@@ -1,7 +1,3 @@
-/*
- * $Id: term.h,v 1.66.2.1 2014/10/01 02:40:29 sfeam Exp $
- */
-
 /* GNUPLOT - term.h */
 
 /*[
@@ -39,8 +35,7 @@
  *   Edit this file depending on the set of terminals you wish to support.
  * Comment out the terminal types that you don't want or don't have, and
  * uncomment those that you want included. Be aware that some terminal
- * types (eg, SUN) will require changes in the makefile
- * LIBS definition.
+ * types will require changes in the makefile LIBS definition.
  */
 
 /*
@@ -54,6 +49,9 @@
  * pslatex and epslatex support is now provided by the combination of
  * post.trm and pslatex.trm.  You cannot build pslatex without post.
  * Both drivers are selected by default, but you can disable them below.
+ *
+ * Enhanced text support is pretty much required for all terminals now.
+ * If you build without GP_ENH_EST text layout will be degraded.
  */
 #define GP_ENH_EST 1		/* estimate string length of enhanced text */
 #define POSTSCRIPT_DRIVER 1	/* include post.trm */
@@ -63,6 +61,16 @@
 #define POSTSCRIPT_DRIVER
 #endif
 
+# ifdef GP_ENH_EST
+#  include "estimate.trm"	/* used for enhanced text processing */
+# endif
+
+/* Unicode escape sequences (\U+hhhh) are handling by the enhanced text code.
+ * Various terminals check every string to see whether it needs enhanced text
+ * processing. This macro allows them to include a check for the presence of
+ * unicode escapes.
+ */
+#define contains_unicode(S) strstr(S, "\\U+")
 
 /* Define SHORT_TERMLIST to select a few terminals. It is easier
  * to define the macro and list desired terminals in this section.
@@ -71,10 +79,6 @@
 #ifdef SHORT_TERMLIST
 # include "dumb.trm"		/* dumb terminal */
 
-# ifdef GP_ENH_EST
-#  include "estimate.trm"	/* used for enhanced text processing */
-# endif
-
 # ifdef POSTSCRIPT_DRIVER
 #  ifdef  PSLATEX_DRIVER
 #   undef PSLATEX_DRIVER
@@ -88,7 +92,7 @@
 # ifdef OS2
 #  include "pm.trm"		/* OS/2 Presentation Manager */
 # endif
-# ifdef _Windows
+# ifdef _WIN32
 #  include "win.trm"		/* MS-Windows */
 # endif
 #else /* include all applicable terminals not commented out */
@@ -102,53 +106,23 @@
 # include "be.trm"
 #endif
 
-
-/****************************************************************************/
-/* MS-DOS and Windows */
-#if defined(MSDOS) || defined(_Windows)
-
-/* MSDOS with emx-gcc compiler */
-# if defined(MSDOS) && defined(__EMX__)
-   /* Vesa-Cards */
-#  define EMXVESA
-#  include "emxvga.trm"
-# endif				/* MSDOS && EMX */
-
-/* MSDOS with djgpp compiler */
-# if defined(DJGPP) && (!defined(DJSVGA) || (DJSVGA != 0))
-#  include "djsvga.trm"
-# endif
-
-/* All other Compilers */
-# ifndef _Windows
-#  ifdef PC
-/* uncomment the next line to include SuperVGA support */
-#   define BGI_NAME "svga256"	/* the name of the SVGA.BGI for Borland C */
-/* this also triggers the inclusion of Super VGA support */
-#   include "pc.trm"		/* all PC types except MS WINDOWS */
-#  endif
-# else				/* _Windows */
-#  include "win.trm"		/* MS-Windows */
-# endif				/* _Windows */
-#endif /* MSDOS || _Windows */
-/****************************************************************************/
-
-
-/* NeXT */
-#ifdef NEXT
-# include "next.trm"
+/* MSDOS with djgpp compiler or _WIN32/Mingw or X11 */
+#if (defined(DJGPP) && (!defined(DJSVGA) || (DJSVGA != 0))) || defined(HAVE_GRX)
+# include "djsvga.trm"
 #endif
 
-/* Apple Mac OS X Server 1.0 (Openstep Unix) */
-/* Apparently, Openstep code won't work on newer versions of
- * MacOS X. If someone can fix this, and provide a proper
- * configure test, let us know.
- */
-/*
- * #if defined(__APPLE__) && defined(__MACH__)
- * # include "openstep.trm"
- * #endif
-*/
+/****************************************************************************/
+/* MSDOS with OpenWatcom compiler */
+#if defined(MSDOS) && defined(__WATCOMC__)
+# include "pc.trm"
+#endif
+/****************************************************************************/
+
+/* Windows */
+#ifdef _WIN32
+/* Windows GDI/GDI+/Direct2D */
+# include "win.trm"
+#endif
 
 /* Apple Mac OS X */
 #ifdef HAVE_FRAMEWORK_AQUATERM
@@ -160,10 +134,6 @@
 #ifdef OS2
 /* presentation manager */
 # include "pm.trm"
-# ifdef EMXVESA
-/* works with DOS and OS/2 (windowed/full screen) */
-#  include "emxvga.trm"
-# endif
 #endif /* OS2 */
 
 
@@ -171,30 +141,16 @@
 /* Terminals for various Unix platforms                                    */
 /***************************************************************************/
 
-/* Linux VGA */
-#ifdef LINUXVGA
-# include "linux.trm"
+/* none */
 
-/* Linux VGAGL */
-# if defined(VGAGL) && defined (THREEDKIT)
-#  include "vgagl.trm"
-# endif
-#endif /* LINUXVGA */
-
-/* SunView */
-#ifdef SUN
-# include "sun.trm"
-#endif
-
-
-/* VAX Windowing System requires UIS libraries */
-#ifdef UIS
-# include "vws.trm"
-#endif
 
 /****************************************************************************/
 /* Terminals not relevant for MSDOS, MS-Windows */
-#if !(defined(MSDOS) || defined(_Windows))
+#if !(defined(MSDOS) || defined(_WIN32))
+
+/* This is not really a terminal, it generates a help section for */
+/* using the linux console. */
+# include "linux-vgagl.trm"
 
 /* gpic for groff */
 #ifdef HAVE_GPIC
@@ -202,18 +158,20 @@
 #endif
 
 /* REGIS graphics language */
-#ifdef VMS
+#if defined(HAVE_REGIS)
 # include "regis.trm"
 #endif
 
-/* Tektronix 4106, 4107, 4109 and 420x terminals */
-# include "t410x.trm"
+#ifdef WITH_TEKTRONIX
+    /* Tektronix 4106, 4107, 4109 and 420x terminals */
+#   include "t410x.trm"
 
-/* a Tek 4010 and others including VT-style */
-# include "tek.trm"
+    /* a Tek 4010 and others including VT-style */
+#   include "tek.trm"
+#endif
 
 
-#endif /* !MSDOS && !_Windows */
+#endif /* !MSDOS && !_WIN32 */
 /****************************************************************************/
 
 
@@ -229,17 +187,21 @@
 /* obsolete: use 'set term postscript level1 */
 /* #include "ai.trm" */
 
-/* HTML Canvas terminal */
+/* Support routines requiring gdlib or cairo */
 #if (defined(HAVE_GD_PNG) || defined(HAVE_CAIROPDF))
 # include "write_png_image.c"
+# define HAVE_KITTY_TERM
+# include "kitty.c"
 #endif
+
+/* HTML Canvas terminal */
 #include "canvas.trm"
 
 /* Computer Graphics Metafile (eg ms office) */
 #include "cgm.trm"
 
 /* CorelDraw! eps format */
-#include "corel.trm"
+/* #include "corel.trm"  */
 
 /* debugging terminal */
 #ifdef DEBUG
@@ -254,7 +216,17 @@
 # include "caca.trm"
 #endif
 
-/* DXF format for use with AutoCad (Release 10.x) */
+/* pseudo-graphics using block or Braille characters */
+#ifndef NO_BITMAP_SUPPORT
+# include "block.trm"
+#endif
+
+/* Legacy terminal for export to AutoCad (Release 10.x)
+ * DWGR10 format (1988)
+ * Still included by popular demand although basically untouched for 20+ years.
+ * Someone please update this terminal to adhere to a newer DXF standard!
+ * http://images.autodesk.com/adsk/files/autocad_2012_pdf_dxf-reference_enu.pdf
+ */
 #include "dxf.trm"
 
 /* Enhanced Metafile Format driver */
@@ -303,11 +275,10 @@
 #include "mif.trm"
 #endif
 
-/* Adobe Portable Document Format (PDF) */
-/* NOTE THAT PDF REQUIRES A SEPARATE LIBRARY : see term/pdf.trm */
-#ifdef HAVE_LIBPDF
-# include "pdf.trm"
-#endif
+/* DEPRECATED since 5.0.6
+ * PDF terminal based on non-free library PDFlib or PDFlib-lite from GmbH.
+ */
+/* # include "pdf.trm" */
 
 #if defined(HAVE_GD_PNG) || defined(HAVE_GD_JPEG) || defined(HAVE_GD_GIF)
 # include "gd.trm"
@@ -319,13 +290,15 @@
 #endif
 
 /* QMS laser printers */
-#include "qms.trm"
+/* #include "qms.trm" */
 
 /* W3C Scalable Vector Graphics file */
 #include "svg.trm"
 
 /* x11 tgif tool */
+#ifdef HAVE_TGIF
 #include "tgif.trm"
+#endif
 
 /* tcl/tk with perl extensions */
 #include "tkcanvas.trm"
@@ -360,24 +333,17 @@
 
 #endif /* NO_BITMAP_SUPPORT */
 
-/* TeX related terminals */
-#define EMTEX
-#define EEPIC
+/* TeX related terminals start here */
 
-/* latex and emtex */
-#include "latex.trm"
+
+/* LaTeX2e picture environment */
+#include "pict2e.trm"
 
 /* latex/tex with picture in postscript */
 #ifdef PSLATEX_DRIVER
 #include "pslatex.trm"
 #endif
 
-/* EEPIC-extended LaTeX driver, for EEPIC users */
-#include "eepic.trm"
-
-/* TPIC specials for TeX */
-#include "tpic.trm"
-
 /* LaTeX picture environment with PSTricks macros */
 #include "pstricks.trm"
 
@@ -385,20 +351,39 @@
 #include "texdraw.trm"
 
 /* METAFONT */
+#ifdef WITH_METAFONT
 #include "metafont.trm"
+#endif
 
 /* METAPOST */
+#ifdef WITH_METAPOST
 #include "metapost.trm"
+#endif
 
 /* ConTeXt */
 #include "context.trm"
 
-#ifdef USE_GGI_DRIVER
-# include "ggi.trm"
+/*
+ * DEPRECATED latex terminals no longer built by default
+ */
+#if 0
+#  define EMTEX
+#  define EEPIC
+#  define OLD_LATEX_TERMINAL
+
+#  include "latex.trm"	/* latex and emtex */
+#  include "eepic.trm"	/* EEPIC-extended LaTeX driver */
+#  include "tpic.trm"	/* TPIC specials for TeX */
+#else
+#  include "latex_old.h" /* deprecation notice for docs */
+#  undef OLD_LATEX_TERMINAL
 #endif
 
-#ifdef GP_ENH_EST
-#include "estimate.trm"
+/* End of TeX related terminals */
+
+
+#ifdef USE_GGI_DRIVER
+# include "ggi.trm"
 #endif
 
 /* WXWIDGETS */
@@ -410,6 +395,11 @@
 # include "cairo.trm"
 #endif
 
+/* webp must come after cairo */
+#ifdef HAVE_WEBP
+# include "webp.trm"
+#endif
+
 #ifdef HAVE_LUA
 #include "lua.trm"
 #endif
