@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: version.c,v 1.111.2.9 2016/02/04 07:27:52 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - version.c */
 
 /*[
@@ -39,14 +35,15 @@ static char *RCSid() { return RCSid("$Id: version.c,v 1.111.2.9 2016/02/04 07:27
 #include "syscfg.h"		/* for FAQ_LOCATION */
 
 
-const char gnuplot_version[] = "5.0";
-const char gnuplot_patchlevel[] = "3";
+const double gnuplot_ver = 6.0;
+const char gnuplot_version[] = "6.0";
+const char gnuplot_patchlevel[] = "0";
 #ifdef DEVELOPMENT_VERSION
 #include "timestamp.h"
 #else
-const char gnuplot_date[] = "2016-02-04 ";
+const char gnuplot_date[] = "2023-12-09 ";
 #endif
-const char gnuplot_copyright[] = "Copyright (C) 1986-1993, 1998, 2004, 2007-2016";
+const char gnuplot_copyright[] = "Copyright (C) 1986-1993, 1998, 2004, 2007-2023";
 
 const char faq_location[] = FAQ_LOCATION;
 
